ANNOTATIONS = 'annotations'
ANNOTATION_OBJECTS = 'annotationObjects'
ATTRIBUTES = 'attributes'
ATTRIBUTE_VALUES = 'attributeValues'
BBOX = 'bbox'
BOX_BOUNDARIES_AND_DIMENSIONS = 'boxBoundariesAndDimensions'
CATEGORIES = 'categories'
CREATED_AT = 'createdAt'
UPDATED_AT = 'updatedAt'
COLOR = 'color'
CONFIDENCE = 'confidence'
DATA = 'data'
DESCRIPTION = 'description'
H = 'h'
ID = 'id'
IS_NORMALIZED = 'isNormalized'
DEST_PROJECT_ID = 'destinationProjectId'
IMAGE = 'image'
IMAGES = 'images'
IMG_FILES = 'imgFiles'
IS_USER_ANNOTATED = 'isUserAnnotated'
KEY = 'key'
LABEL = 'label'
LABEL_COLOR = 'labelColor'
LABEL_REF = 'labelRef'
LABEL_TEXT = 'labelText'
LABELS = 'labels'
LINE = 'line'
METADATA = 'metadata'
META_UPDATES_ARRAY = 'metaUpdatesArray'
NAME = 'name'
OBJECT_KEY = 'objectKey'
POINTS = 'points'
POLYGON = 'polygon'
REF = 'ref'
SHAPE_ID = 'shapeId'
SUPERCATEGORY = 'supercategory'
TEXTS = 'texts'
TYPE = 'type'
USERNAME = 'userName'
VALUE_TEXT = 'valueText'
VALUES = 'values'
VALUE = 'value'
W = 'w'
X = 'x'
Y = 'y'
SESSION_ID = "sessionId"
TOTAL_IMAGE_COUNT = "totalImageCount"
UPLOADED_IMAGE_COUNT = "uploadedImageCount"
SELECTION_ID = "selectionId"
USER_ID = "userId"
COLLECTION_NAME = "collectionName"
IS_APPLY_TO_ALL_FILES = "isApplyToAllFiles"
FILES = "files"
OBJECT_TYPE = "objectType"
STORAGE_PATH = "storagePath"
BUCKET_NAME = "bucketName"
STORAGE_NAME = "storageName"


import json
import traceback
import requests


class DatasetInterface:

    def __init__(self, auth_token: str, dalalake_url: str):
        self.auth_token = auth_token
        self.dalalake_url = dalalake_url


    """
    create dataset from search objects
    """
    def create_dataset(self, selection_id, payload_dataset_creation):
        hed = {'Authorization': 'Basic ' + self.auth_token}
        url = f'{self.dalalake_url}/api/client/dataset/create?datasetSelectionId={selection_id}'

        try:
            response = requests.get(url=url, json=payload_dataset_creation, headers=hed)
            status_code = response.status_code
            if status_code == 200:
                return response.json()
            else:
                response = response.json()
                if 'error' in response:
                    if 'message' in response['error']:
                        return {
                            "isSuccess": False,
                            "message": response['error']['message']
                            }
                    else:
                        return {"isSuccess": False}
                else:
                    return {"isSuccess": False}
        #Handle connection error
        except requests.exceptions.ConnectionError as e:
            print("Connection error from Data Lake connection")
            return {"isSuccess": False, "message": "Failed to connect to Data Lake"}
        #Handle timeout error
        except requests.exceptions.Timeout as e:
            print("Timeout error from Data Lake connection")
            return {"isSuccess": False, "message": "Failed to connect to Data Lake"}
        #Handle HTTP errors
        except requests.exceptions.HTTPError as e:
            print("HTTP error from Data Lake connection")
            return {"isSuccess": False, "message": "Failed to connect to Data Lake"} 
        except requests.exceptions.RequestException as e:
            print(f"An unexpected request exception occurred: {format(e)}")
            traceback.print_exc()
            return {"isSuccess": False, "message": "Error in checking version compatibility"}
        except Exception as e1:
            print(f"An unexpected exception occurred: {format(e1)}")
            traceback.print_exc()
            return {"isSuccess": False, "message": "Error in checking version compatibility"}


    """
    update dataset from search objects
    """
    def update_existing_dataset_version(self, selection_id, payload_dataset_update, version_id):
        hed = {'Authorization': 'Basic ' + self.auth_token}
        url = f'{self.dalalake_url}/api/client/dataset/updateExistingVersion?datasetSelectionId={selection_id}&datasetVersionId={version_id}'

        try:
            response = requests.get(url=url, json=payload_dataset_update, headers=hed)
            status_code = response.status_code
            if status_code == 200:
                return response.json()
            else:
                response = response.json()
                if 'error' in response:
                    if 'message' in response['error']:
                        return {
                            "isSuccess": False,
                            "message": response['error']['message']
                            }
                    else:
                        return {"isSuccess": False}
                else:
                    return {"isSuccess": False}
        #Handle connection error
        except requests.exceptions.ConnectionError as e:
            print("Connection error from Data Lake connection")
            return {"isSuccess": False, "message": "Failed to connect to Data Lake"}
        #Handle timeout error
        except requests.exceptions.Timeout as e:
            print("Timeout error from Data Lake connection")
            return {"isSuccess": False, "message": "Failed to connect to Data Lake"}
        #Handle HTTP errors
        except requests.exceptions.HTTPError as e:
            print("HTTP error from Data Lake connection")
            return {"isSuccess": False, "message": "Failed to connect to Data Lake"} 
        except requests.exceptions.RequestException as e:
            print(f"An unexpected request exception occurred: {format(e)}")
            traceback.print_exc()
            return {"isSuccess": False, "message": "Error in checking version compatibility"}
        except Exception as e1:
            print(f"An unexpected exception occurred: {format(e1)}")
            traceback.print_exc()
            return {"isSuccess": False, "message": "Error in checking version compatibility"}


    """
    create dataset version from search objects
    """
    def create_new_dataset_version(self, selection_id, payload_dataset_update, version_id):
        hed = {'Authorization': 'Basic ' + self.auth_token}
        url = f'{self.dalalake_url}/api/client/dataset/updateNewVersion?datasetSelectionId={selection_id}&datasetVersionId={version_id}'

        try:
            response = requests.get(url=url, json=payload_dataset_update, headers=hed)
            status_code = response.status_code
            if status_code == 200:
                return response.json()
            else:
                response = response.json()
                if 'error' in response:
                    if 'message' in response['error']:
                        return {
                            "isSuccess": False,
                            "message": response['error']['message']
                            }
                    else:
                        return {"isSuccess": False}
                else:
                    return {"isSuccess": False}
        #Handle connection error
        except requests.exceptions.ConnectionError as e:
            print("Connection error from Data Lake connection")
            return {"isSuccess": False, "message": "Failed to connect to Data Lake"}
        #Handle timeout error
        except requests.exceptions.Timeout as e:
            print("Timeout error from Data Lake connection")
            return {"isSuccess": False, "message": "Failed to connect to Data Lake"}
        #Handle HTTP errors
        except requests.exceptions.HTTPError as e:
            print("HTTP error from Data Lake connection")
            return {"isSuccess": False, "message": "Failed to connect to Data Lake"} 
        except requests.exceptions.RequestException as e:
            print(f"An unexpected request exception occurred: {format(e)}")
            traceback.print_exc()
            return {"isSuccess": False, "message": "Error in checking version compatibility"}
        except Exception as e1:
            print(f"An unexpected exception occurred: {format(e1)}")
            traceback.print_exc()
            return {"isSuccess": False, "message": "Error in checking version compatibility"}


    """
    delete dataset version
    """
    def delete_dataset_version(self, version_id):
        hed = {'Authorization': 'Basic ' + self.auth_token}
        url = f'{self.dalalake_url}/api/client/dataset/deleteVersion?datasetVersionId={version_id}'

        try:
            response = requests.get(url=url, json={}, headers=hed)
            status_code = response.status_code
            if status_code == 200 or status_code == 204:
                return {"isSuccess": True}
            else:
                response = response.json()
                if 'error' in response:
                    if 'message' in response['error']:
                        return {
                            "isSuccess": False,
                            "message": response['error']['message']
                            }
                    else:
                        return {"isSuccess": False}
                else:
                    return {"isSuccess": False}
        #Handle connection error
        except requests.exceptions.ConnectionError as e:
            print("Connection error from Data Lake connection")
            return {"isSuccess": False, "message": "Failed to connect to Data Lake"}
        #Handle timeout error
        except requests.exceptions.Timeout as e:
            print("Timeout error from Data Lake connection")
            return {"isSuccess": False, "message": "Failed to connect to Data Lake"}
        #Handle HTTP errors
        except requests.exceptions.HTTPError as e:
            print("HTTP error from Data Lake connection")
            return {"isSuccess": False, "message": "Failed to connect to Data Lake"} 
        except requests.exceptions.RequestException as e:
            print(f"An unexpected request exception occurred: {format(e)}")
            traceback.print_exc()
            return {"isSuccess": False, "message": "Error in checking version compatibility"}
        except Exception as e1:
            print(f"An unexpected exception occurred: {format(e1)}")
            traceback.print_exc()
            return {"isSuccess": False, "message": "Error in checking version compatibility"}



import json
from typing import TYPE_CHECKING

from layernext.datalake.constants import MetadataUploadType
from .keys import COLLECTION_NAME, FILES, IMAGES, IS_APPLY_TO_ALL_FILES, METADATA, OBJECT_TYPE

if TYPE_CHECKING:
    from . import DatalakeClient


class Metadata:
    
    def __init__(self, client:"DatalakeClient"):
        self._client = client

    """"
    Uploads metadata to the datalake by passing a json file (load json file and pass it as a parameter)
    """
    def upload_metadata_json(self, storage_base_path: str,object_type:str, file_path: str):
        
        # load json file
        file = open(file_path)
        annotation_data = json.load(file)
        file.close()

        metaData_json_array = annotation_data[FILES]

        if len(metaData_json_array) > 0:

            payload = {
                COLLECTION_NAME: storage_base_path,
                OBJECT_TYPE: object_type,
                METADATA: metaData_json_array
            }

            meta_data_updates = self._client.datalake_interface.upload_metadata(payload, MetadataUploadType.BY_JSON)
            return meta_data_updates
        

    """
    Uploads metadata to the datalake by passing a metadata object
    """
    def upload_metadata_object(
            self,
            collection_name: str,
            object_type: str,
            metadata_object: dict,
            is_apply_to_all_files: bool
    ):
        payload = {
            COLLECTION_NAME: collection_name,
            OBJECT_TYPE: object_type,
            METADATA: metadata_object,
            IS_APPLY_TO_ALL_FILES: is_apply_to_all_files
        }

        meta_data_updates = self._client.datalake_interface.upload_metadata(payload, MetadataUploadType.BY_META_OBJECT)
        return meta_data_updates
                
    



"""
Script to download images and text files to local machine and generate path file
"""
import requests
import json
import os
from multiprocessing.pool import ThreadPool
from zipfile import ZipFile


# # IP+port or url of the server # (eg: https://qa.deepzea.com)
# serverUrl = 'http://localhost:8080' # default

class DatasetSync:
    def __init__(self, encoded_key_secret: str, server_url: str, download_path: str):
        self.failed_downloads_present = False
        self.encoded_key_secret = encoded_key_secret
        self.server_url = server_url
        self.local_download_abs_dir = download_path

    """
    method to get item URL list from layerX (per page)
    @params - callUrl=(Url to get itemlist of given group and version from layernext), payload=(pageNo,pageSize)
    @returns - response=(response from layerX containing URL list)"""
    def get_data_from_server(self, call_url, payload):

        hed = {'Authorization': 'Basic ' + self.encoded_key_secret}

        print(hed)
        print(call_url)
        try:
            call_response = requests.get(call_url, params=payload, headers=hed, timeout=60)
        except (requests.exceptions.ConnectionError, requests.exceptions.ConnectTimeout, requests.exceptions.Timeout) as e :
            print(f'Error connecting to layernext')
            print(e.errno)
            print("We are facing a problem with the network, please retry to download missing items")
            quit()
        print(call_response, flush=True)
        response = call_response.json()
        return response

    """
    Method to download files
    @params: fileData=(data including url of file), downloadLocation=(file path to save downloaded file) """
    def download_file(self, file_data, download_location):
        # download files from newly added paths
        # print('Downloading file '+fileData["fileName"])
        r = requests.get(file_data["fileUrl"], timeout=25)
        download_path = download_location
        # print(downloadLocation)
        with open(download_path, 'wb') as f:
            f.write(r.content)

    """
     handle download for a single dataset item (image and textfile)
     @params: arg_list=[fileData=(data of file), identifier=(unique identifier name for the dataset version)]
     @returns: {False<when download failed>, arg_list<from params>}
           or  {
                   True<when download success>,
                   {textFileImagePathData={imagePath=(file path of downloaded item), pathFileName=("training path file" name - stores paths of downloaded images used for training)}}
               } """
    def handle_one_download(self, arg_list):
        file_data = arg_list[0]
        # //print(fileData)
        # identifier = arg_list[1]
        # ---------------------

        

        formatted_write_key = file_data['fileName']
        file_path = f'{file_data["fileDownloadFolderLocation"]}/' + file_data['fileName']

        # if item have to be extracted (a zip file)    
        # if "isUnzipRequired" in file_data:
        #     folder_path_absolute = os.path.abspath(f'./{self.data_directory_name}/' + file_path)
        #     file_path = f'{file_path}.zip'

            # if self.local_download_abs_dir != None:
                # absPath = os.path.abspath(f'./{self.data_directory_name}/' + file_path)
                # folder_path_absolute = os.path.abspath(os.path.join(self.local_download_abs_dir, self.data_directory_name,file_path))

        if self.local_download_abs_dir != None:

            # if item have to be extracted (a zip file)    
            if "isUnzipRequired" in file_data:
                folder_path_absolute = os.path.join(self.local_download_abs_dir, self.data_directory_name, file_path)
                file_path = f'{file_path}.zip'
            #If custom path is given instead of current directory
            _file_path = os.path.join(self.local_download_abs_dir+f'/{self.data_directory_name}/' + file_path)
            _folder_path_absolute = os.path.abspath(_file_path)
            file_path = os.path.join(self.local_download_abs_dir, _folder_path_absolute)
            file_path_absoulute = file_path
        else:
            # if item have to be extracted (a zip file)    
            if "isUnzipRequired" in file_data:
                folder_path_absolute = os.path.abspath(f'./{self.data_directory_name}/' + file_path)
                file_path = f'{file_path}.zip'
            file_path_absoulute = os.path.abspath(f'./{self.data_directory_name}/' + file_path)



        if "writingPathFileName" in file_data:
            # write to a annotation file path list
            path_file_name = file_data["writingPathFileName"]
            text_file_image_path_data = (file_path_absoulute, path_file_name)
        else:
            text_file_image_path_data = None

        if "rawAnnotations" in file_data:
            # append raw annotations to json
            self.raw_data_arr.append(file_data["rawAnnotations"])

        if(not os.path.exists(file_path_absoulute)):
            # download files
            try:
                if "fileUrl" in file_data:
                    self.download_file(file_data, file_path_absoulute)  # download image
            except Exception as e:
                print(f'Failed downloading - {formatted_write_key}')
                print(e)
                return (False, arg_list)

            # if item have to be extracted (a zip file)    
            if "isUnzipRequired" in file_data:
                with ZipFile(file_path_absoulute, 'r') as zObject:
                    zObject.extractall(path=folder_path_absolute)
                os.remove(file_path_absoulute)
                

            # If item is not available in the syncdatafile (file hasn't downloaded before)
            print('Downloded item - ' + formatted_write_key)

            # return fileName and textFileName to update the textfile
            return (True,  text_file_image_path_data)

        else:
            print('Item already exists, OK ', file_data['fileName'])
            return (True, text_file_image_path_data)

    """
    main method to download items and update datafiles
    @params: dataList=(response from layerX containing URL list) """
    def download_page(self, data_list):
        arg_list = []
        for val in data_list['resourceArray']:
            arg_list.append([val, data_list['identifier']])

        # Download items in parallel
        print("starting page download")
        with ThreadPool(10) as p:
            for res in p.imap(self.handle_one_download, arg_list):
                is_downloded = res[0]
                download_data = res[1]
                if not is_downloded:
                    # try again to download the failed download
                    print(f"Retrying Download - {download_data[0]['fileName']}")
                    res = self.handle_one_download(download_data)
                    is_downloded = res[0]
                    download_data = res[1]

                if is_downloded:
                    #if file is downloaded, write it to master csv
                    text_file_image_path_data = download_data

                    if text_file_image_path_data is not None:
                        # write image file path to train text file
                        image_path = text_file_image_path_data[0]
                        path_file_name = text_file_image_path_data[1]
                        text_path_file_location = os.path.join(self.data_directory_name, f'{path_file_name}.txt')
                        image_path_location = os.path.join(self.data_directory_name, text_file_image_path_data[0])
                        if self.local_download_abs_dir != None:
                            text_path_file_location = os.path.join(self.local_download_abs_dir, self.data_directory_name, f'{path_file_name}.txt')
                        with open(text_path_file_location, 'a+', newline='') as train_txt_file:
                            train_txt_file.write(image_path_location + "\n")
                        p.close()
                else:
                    print(f"Retrying Failed! - {download_data[0]['fileName']}")
                    self.failed_downloads_present = True
                    p.close()

        print("page download done")

    """
    Method to create required directories for file download """
    def initiate_download(self, response, page_no):
        print("data format: ", response['format'])

        # global self.dataDirectoryName  # folder to download image and text files
        # create a folder with dataset groupname
        self.data_directory_name = response['groupUniqueName']
        data_directory_path = f'./{self.data_directory_name}'
        if self.local_download_abs_dir != None:
            #If custom path is given for download
            data_directory_path = os.path.join(self.local_download_abs_dir, data_directory_path)

        if page_no == 1:
            print('1st page')
            # create required files and directories
            if not os.path.exists(data_directory_path):
                os.makedirs(data_directory_path)
                print(f"Created {self.data_directory_name} folder")
            
            # get raw json file name (if applicable)
            if "rawFileName" in response:
                self.json_file_name = response["rawFileName"]

        # create required directories
        for directory in response["creatableDirectories"]:
            dir_path = os.path.join(data_directory_path, directory)
            if not os.path.exists(dir_path):
                os.makedirs(dir_path)
                print(f"Created folder - {dir_path}")

        # updateVersionData(response)
        self.download_page(response)

    """
    method to get url list from server and execute the main script for all pages
    @params - callUrl (Url to get itemlist of given group and version from layernext), pageNo 
    @params - status_list, is_annotated_only - use only for download annotation from annotation studio project
    """
    def get_all_pages(
        self, 
        call_url, 
        page_no, 
        status_list = [], 
        is_annotated_only = False, 
        project_id_list = [], 
        operation_id_list = [],
        annotation_type = "all"):

        payload = {
            'pageNo': page_no, 
            'pageSize': 250, 
            'statusList': json.dumps(status_list), 
            "isAnnotatedOnly": is_annotated_only,
            "projectIdList": json.dumps(project_id_list),
            "operationIdList": json.dumps(operation_id_list),
            "annotationType": annotation_type
        }
        response = self.get_data_from_server(call_url, payload)
        print('Downloading page no: ' + str(page_no))
        if "data" in response and "identifier" in response["data"]:
            response_data = response["data"]
            self.json_file_name = response_data["rawFileName"]
            # Direct the response to download files
            self.initiate_download(response_data, page_no)

            # recursively call this again to get next page
            if(response_data['nextPage'] == True):
                next_page_no = int(page_no)+1
                self.get_all_pages(call_url, next_page_no, status_list, is_annotated_only, project_id_list, operation_id_list, annotation_type)
            print("Download Complete")
            if self.failed_downloads_present:
                print(
                    "Unfortunately we failed to download everything, please retry to download missing items")

        else:
            print("No data recieved from remote for given group and/or version")

    # main script starter
    """
    Download dataset
    From dataset manager
    @param version_id - id of dataset version 
    @param export_type - dataset export format """
    def download_dataset(self, version_id: str, export_type: str):
        # get dataset url data from server
        _base_url = f'{self.server_url}/dataset/api/dataset/getVersionData/'
        _call_url = f'{_base_url}{version_id}/{export_type}/'
        # start operation
        self.get_all_pages(_call_url, 1)
        # quit()

    # main script starter
    """
    Download collection annotations
    From datalake - Both ground trugh
    @param collection_id - id of dataset version  
    @param model_id - Optional: id of the model (same operation_id given in upload annotations) 
    if we need annotations for that specific model """
    def download_collection(self, collection_id: str, annotation_type, operation_id_list):

        if annotation_type is None:
            annotation_type = 'all'

        if operation_id_list is None:
            operation_id_list = []

        if annotation_type != 'all' and annotation_type != 'human' and annotation_type != 'machine':
            print("invalid annotation_type. annotation_type must be 'all', 'human', or 'machine' ")
            return {
                "isSuccess" : False,
                "error" : "annotation_type"
            }

        # get collection url data from server
        _base_url = f'{self.server_url}/datalake/api/client/getAnnotations/'
        #Pass the model_id only if specified
        # if model_id is None:
        _call_url = f'{_base_url}{collection_id}/collection'
        # else:
        #     _call_url = f'{_base_url}{collection_id}/collection?operationId={model_id}'

        # define annotation json object 
        self.raw_data_arr = []
        self.json_file_name = 'default_json_filename.json'
        
        # start operation
        self.get_all_pages(_call_url, 0, None, None, None, operation_id_list, annotation_type)

        # save json (anno list) to a file
        _raw_annotations = {
            "images": self.raw_data_arr
        }
        json_raw_annotations = json.dumps(_raw_annotations, indent=2)
        _file_path = os.path.join(self.data_directory_name, self.json_file_name)

        if self.local_download_abs_dir != None:
            #custom path is given instead of current directory
            _file_path = os.path.join(self.local_download_abs_dir, _file_path)
            
        with open(_file_path, "w") as outfile:
            outfile.write(json_raw_annotations)
        print("Created RAW annotation file")
        # quit()


    """
    Download project annotations
    From datalake - Ground truth data
    @param project_id 
    """
    def download_project_v2(self, project_id_list, status_list, is_annotated_only):

        # get collection url data from server
        _call_url = f'{self.server_url}/studio/api/client/projects/getAnnotations'
       
        # define annotation json object 
        self.raw_data_arr = []
        self.json_file_name = 'default_json_filename.json'
        
        # start operation
        self.get_all_pages(_call_url, 0, status_list, is_annotated_only, project_id_list)

        # save json (anno list) to a file
        _raw_annotations = {
            "images": self.raw_data_arr
        }
        json_raw_annotations = json.dumps(_raw_annotations, indent=2)
        _file_path = os.path.join(self.data_directory_name, self.json_file_name)
        if self.local_download_abs_dir != None:
            #custom path is given instead of current directory
            _file_path = os.path.join(self.local_download_abs_dir, _file_path)

        with open(_file_path, "w") as outfile:
            outfile.write(json_raw_annotations)
        print("Created RAW annotation file")


# sample run command:
# python3 sync.py <url> <version ID> <format type> <access token>



import traceback

from layernext.datalake.constants import MULTI_PART_UPLOAD_CHUNK_SIZE

import requests


class StorageInterface:

    def __init__(self, storage_type, url_list):
        self.e_tag = ""
        #Create client based on storage type
        if(storage_type == "AWS_S3"):
            from layernext.datalake.aws_s3_client import AWSS3Client
            self.storage_client = AWSS3Client()
        elif(storage_type == "GCP"):
            from layernext.datalake.gcs_client import GCSClient
            self.storage_client = GCSClient()
        elif(storage_type == "AZURE"):
            from layernext.datalake.azure_client import AzureClient
            self.storage_client = AzureClient(url_list)
        else:
            raise Exception("Invalid storage type: " + storage_type)

        

    ''''
    Upload file to the storage bucket (common code for AWS/Google/Azure)
    '''

    def upload_to_storage(self, url, path, content_type, chunk_id, file_size, start_byte = 0, read_bytes = MULTI_PART_UPLOAD_CHUNK_SIZE):
        try:
            with open(path, 'rb') as object_file:
                # Read chunk of file from start_byte
                object_file.seek(start_byte)
                file_data = object_file.read(read_bytes)
                #print('Uploading ' + str(read_bytes) + ' bytes from ' + str(start_byte))
                content_range = f'bytes {start_byte}-{start_byte+read_bytes-1}/{file_size}'
                #url, chunk_id, chunk_data, content_type, content_range, read_bytes
                return self.storage_client.upload_chunk(url, chunk_id, file_data, content_type, content_range, read_bytes)
                
        #Handle connection error
        except requests.exceptions.ConnectionError as e:
            print("Connection error occurred in upload")
            return {"isSuccess": False}
        #Handle timeout error
        except requests.exceptions.Timeout as e:
            print("Timeout error occurred in upload")
            return {"isSuccess": False}
        #Handle HTTP errors
        except requests.exceptions.HTTPError as e:
            print("HTTP error occurred in upload")
            return {"isSuccess": False} 
        #Handle other errors
        except requests.exceptions.RequestException as e:
            print("An exception occurred in upload")
            traceback.print_exc()
            return {"isSuccess": False}

        except Exception as e1:
            print("An exception occurred in upload")
            traceback.print_exc()
            return {"isSuccess": False}



# System Label Type
from enum import Enum

LABEL_CLASS = 1
LABEL_CLASS_WITH_ATTRIBUTES = 2
FILE_UPLOAD_THREADS = 10
SUB_FILE_LENGTH = 100
# Annotation Geometric Type
LINE_ANNOTATION = 'line'
POLYGON_ANNOTATION = 'polygon'
BOX_ANNOTATION = 'rectangle'

# Operation Data Meta Updates
OPERATION_TYPE_ANNOTATION = 1
OPERATION_MODE_HUMAN = 1
OPERATION_MODE_AUTO = 2

# Request Batch Sizes
META_UPDATE_REQUEST_BATCH_SIZE = 1000

#Chunk size in multi-part upload: 100MB
MULTI_PART_UPLOAD_CHUNK_SIZE = 100 * 1024 * 1024

MAX_UPLOAD_RETRY_COUNT = 20

class MediaType(Enum):
    VIDEO = 4
    IMAGE = 5
    OTHER = 7

class ObjectType(Enum):
    VIDEO = 1
    IMAGE = 2
    DATASET = 3
    VIDEO_COLLECTION = 4
    IMAGE_COLLECTION = 5

class JobStatus(Enum):
    IN_PROGRESS = 1
    COMPLETED = 2
    QUEUED = 3
    FAILED = 4

class MetadataUploadType(Enum):
    BY_JSON = 1
    BY_META_OBJECT = 2

class AnnotationUploadType(Enum):
    BY_FILE_NAME = 1
    BY_STORAGE_PATH = 2
    BY_UNIQUE_NAME = 3
    BY_FILE_NAME_OR_UNIQUE_NAME = 4


import json
from typing import TYPE_CHECKING

from .annotation import Annotation
from .constants import OPERATION_MODE_HUMAN, AnnotationUploadType
from .keys import CATEGORIES
from .label import Label
from .logger import get_debug_logger

if TYPE_CHECKING:
    from . import DatalakeClient

ground_truth_logger = get_debug_logger('GroundTruth')


class GroundTruth(Annotation):
    def __init__(self, client: "DatalakeClient"):
        super().__init__(client)

    def upload_groundtruth_json(
            self,
            storage_base_path: str,
            operation_id: str,
            file_path: str,
            annotation_geometry: str,
            is_normalized: bool,
            dest_project_id: str,
            version: str,
            bucket_name: str,
            upload_type: AnnotationUploadType,
    ):
        ground_truth_logger.debug(f'Started uploading ground truth data for model_id: {operation_id}')
        self.upload_annotation_json(
            storage_base_path,
            operation_id,
            file_path,
            annotation_geometry,
            OPERATION_MODE_HUMAN,
            is_normalized,
            dest_project_id,
            version,
            bucket_name,
            upload_type,
        )
        ground_truth_logger.debug(f'Finished uploading ground truth data for model_id: {operation_id}')

    def upload_coco(self, file_path: str):
        # load json file
        f = open(file_path)
        coco_data = json.load(f)
        f.close()

        label = Label(client=self._client)
        label.create_label_from_cocojson(categories=coco_data[CATEGORIES])
        # TODO: format Images and Annotations as MetaUpdates format, then call metaUpdates API




import base64
import re
import time

from layernext.datalake.annotation import Annotation
from layernext.datalake.metadata import Metadata
from layernext.datalake.query import Query

from .ground_truth import GroundTruth
from .constants import BOX_ANNOTATION, POLYGON_ANNOTATION, LINE_ANNOTATION, AnnotationUploadType, MediaType, JobStatus
from .datalakeinterface import DatalakeInterface
from .file_upload import FileUpload
from .file_trash import FileTrash
from .logger import get_debug_logger
from .model_run import ModelRun

datalake_logger = get_debug_logger('DatalakeClient')


class DatalakeClient:
    """
    Python SDK of Datalake
    """

    def __init__(self, encoded_key_secret: str, layernext_url: str) -> None:
        _datalake_url = f'{layernext_url}/datalake' 
        #_datalake_url = f'{layernext_url}'
        self.datalake_interface = DatalakeInterface(encoded_key_secret, _datalake_url)

    def check_sdk_version_compatibility(self, sdk_version: str):
        """
        check sdk version compatibility
        """

        if re.compile(r'^(\d+\.)+\d+$').match(sdk_version) is None:
            raise Exception('sdk_version is invalid format')

        if sdk_version is None or sdk_version == '' :
            raise Exception('sdk_version is None')
        
        res = self.datalake_interface.check_sdk_version_compatibility(sdk_version)

        if res["isCompatible"] == False:
            raise Exception(res["message"])

    def upload_annotation_from_cocojson(self, file_path: str):
        """
        available soon
        """
        datalake_logger.debug(f'file_name={file_path}')
        _annotation = GroundTruth(client=self)
        _annotation.upload_coco(file_path)

    def upload_modelrun_from_json(
            self,
            storage_base_path: str,
            model_id: str,
            file_path: str,
            annotation_geometry: str,
            is_normalized: bool,
            dest_project_id: str,
            version: str,
            bucket_name: str,
            upload_type: AnnotationUploadType

    ):
        datalake_logger.debug(f'upload_modelrun_from_json file_path={file_path}, '
                              f'annotation_geometry={annotation_geometry}')
        _model = ModelRun(client=self)
        if annotation_geometry == BOX_ANNOTATION:
            _model.upload_modelrun_json(storage_base_path, model_id, file_path, BOX_ANNOTATION, is_normalized, dest_project_id, version, bucket_name, upload_type)
        elif annotation_geometry == POLYGON_ANNOTATION:
            _model.upload_modelrun_json(storage_base_path, model_id, file_path, POLYGON_ANNOTATION, is_normalized, dest_project_id, version, bucket_name, upload_type)
        elif annotation_geometry == LINE_ANNOTATION:
            _model.upload_modelrun_json(storage_base_path, model_id, file_path, LINE_ANNOTATION, is_normalized, dest_project_id, version, bucket_name, upload_type)
        elif annotation_geometry is None:
            _model.upload_modelrun_json(storage_base_path, model_id, file_path, None, is_normalized, dest_project_id, version, bucket_name, upload_type)
        else:
            datalake_logger.debug(f'unsupported annotation_geometry={annotation_geometry}')

    def upload_groundtruth_from_json(
            self,
            storage_base_path: str,
            operation_id: str,
            file_path: str,
            annotation_geometry: str,
            is_normalized: bool,
            dest_project_id: str,
            version: str,
            bucket_name: str,
            upload_type: AnnotationUploadType
    ):
        datalake_logger.debug(f'upload_groundtruth_from_json file_path={file_path}, '
                              f'annotation_geometry={annotation_geometry}')
        _groundTruth = GroundTruth(client=self)
        if annotation_geometry == BOX_ANNOTATION:
            _groundTruth.upload_groundtruth_json(storage_base_path, operation_id, file_path, BOX_ANNOTATION, is_normalized, dest_project_id, version, bucket_name, upload_type)
        elif annotation_geometry == POLYGON_ANNOTATION:
            _groundTruth.upload_groundtruth_json(storage_base_path, operation_id, file_path, POLYGON_ANNOTATION, is_normalized, dest_project_id, version, bucket_name, upload_type)
        elif annotation_geometry == LINE_ANNOTATION:
            _groundTruth.upload_groundtruth_json(storage_base_path, operation_id, file_path, LINE_ANNOTATION, is_normalized, dest_project_id, version, bucket_name, upload_type)
        elif annotation_geometry is None:
            _groundTruth.upload_groundtruth_json(storage_base_path, operation_id, file_path, None, is_normalized, dest_project_id, version, bucket_name, upload_type)
        else:
            datalake_logger.debug(f'unsupported annotation_geometry={annotation_geometry}')

    def file_upload(self, path: str, collection_type, collection_name, meta_data_object, application_code, meta_data_override, storage_prefix_path):
        _upload = FileUpload(client=self, application_code=application_code)
        upload_res = _upload.file_upload_initiate(path, collection_type, collection_name, meta_data_object, meta_data_override, storage_prefix_path)
        return upload_res

    def get_upload_status(self, collection_name):
        _upload = FileUpload(client=self)
        return _upload.get_upload_status(collection_name)

    def remove_collection_annotations(self, collection_id: str, model_run_id: str):
        print(f'annotation delete of collection ={collection_id}', f'model id={model_run_id}')
        _model = Annotation(client=self)
        _model.remove_collection_annotations(collection_id, model_run_id)


    """
    get selection id for query, collection id, filter data
    """
    def get_selection_id(self, collection_id, query, filter, object_type, object_list, is_all_selected=True):
        _query = Query(client=self)
        response = _query.get_selection_id(collection_id, query, filter, object_type, object_list, is_all_selected)
        return response


    def get_object_type_by_id(self, object_id):
        response = self.datalake_interface.get_object_type_by_id(object_id)
        return response

    def get_system_labels(self, group_id = None):
        response = self.datalake_interface.get_all_label_list(group_id)
        return response

    def attach_labels_to_group(self, group_id, label_keys):
        if group_id == '' or len(label_keys) == 0:
            print('Label group id or label list is empty')
            return { "is_success": False}
        response = self.datalake_interface.add_labels_to_group(group_id, label_keys)
        return response

    def detach_labels_from_group(self, group_id, label_keys):
        if group_id == '' or len(label_keys) == 0:
            print('Label group id or label list is empty')
            return { "is_success": False}
        response = self.datalake_interface.remove_labels_from_group(group_id, label_keys)
        return response

    def get_all_label_groups(self):
        response = self.datalake_interface.get_all_group_list()
        return response

    def create_label_group(self, group_name, label_keys):
        if group_name == '' or len(label_keys) == 0:
            print('Label group name or label list is empty')
            return None
        response = self.datalake_interface.create_label_group(group_name, label_keys)
        return response

    def wait_for_job_complete(self, job_id):
        print(f"Waiting until complete the job: {job_id}")
        while True:
            try:
                job_detils = self.datalake_interface.check_job_status(job_id)

                if job_detils["isSuccess"]:
                    job_status = job_detils["status"]
                    job_progress = job_detils["progress"]
                    print(f'Job progress: {job_progress}')
                    if job_status == JobStatus.COMPLETED.value:
                        res = {
                            "is_success" : True,
                            "job_status" : "COMPLETED"
                        }
                        print(res)
                        return res
                    elif job_status == JobStatus.FAILED.value:
                        res = {
                            "is_success" : True,
                            "job_status" : "COMPLETED"
                        }
                        print(res)
                        return res
                    else:
                        time.sleep(30)
                else:
                    res = {
                        "is_success": False,
                        "job_status": "FAILED"
                    }
                    print(res)
                    return res
            except Exception as e:
                print(f"An exception occurred: {format(e)}")
                res = {
                    "is_success": False,
                    "job_status": "FAILED"
                }
                print(res)
                return res
    
    """
    trash selection object
    """
    def trash_datalake_object(self, selection_id):
        _trash = FileTrash(client=self)
        return _trash.trash_files(selection_id)

    def get_file_download_url(self, file_key):
        return self.datalake_interface.get_file_download_url(file_key)
    
    """
    upload metadata by using json file
    """
    def upload_metadata_from_json(
            self,
            storage_base_path: str,
            object_type: str,
            file_path: str,
    ):
        _metadata = Metadata(client=self)
        response =  _metadata.upload_metadata_json(storage_base_path,object_type, file_path)
        print(response)
        return response

    """
    upload metadata by using meta object
    """
    def upload_metadata_from_metaObject(
            self,
            collection_name: str,
            object_type: str,
            metadata_object: dict,
            is_apply_to_all_files: bool
    ):
        _metadata = Metadata(client=self)
        response = _metadata.upload_metadata_object(collection_name,object_type, metadata_object, is_apply_to_all_files)
        print(response)
        return response
    


import datetime
from typing import TYPE_CHECKING
from .logger import get_debug_logger

if TYPE_CHECKING:
    from . import DatalakeClient

annotation_logger = get_debug_logger('Query')


class Query:
    def __init__(self, client: "DatalakeClient"):
        self._client = client


    def get_selection_id(self, collection_id, query, filter, object_type, object_list, is_all_selected):
        if object_list == None:
            object_list = []
        filterData = {
            "contentType": object_type,
            "filterBy": [],
            "date": None
        }

        if filter != None:
            if ("filterBy" in filter and len(filter["filterBy"]) > 0):
                filterData["filterBy"] = filter["filterBy"]
            else:
                filterData["filterBy"] = []

            if ("date" in filter):
                filterData["date"] = filter["date"]
        
        #"metadata.Tags=2.1"
        payload = {
            "isAllSelected": is_all_selected,
            # "projectType": 3,
            "objectIdList": object_list,
            "filterData": filterData,
            "query": query,
            "collectionId": collection_id
        }
        print(payload)
        response = self._client.datalake_interface.get_selection_id(payload)
        return response


import requests
from layernext.datalake.storage_client import StorageClient


class AWSS3Client(StorageClient):
    def upload_chunk(self, url, chunk_id, chunk_data, content_type, content_range, read_bytes):

        headers={'Content-Type': content_type}
        res = requests.put(url, data=chunk_data, headers=headers)
        if(res.status_code != 200):
            print("Error in uploading file to S3: Status code: " + str(res.status_code))
            print(res.text)
            return {"isSuccess": False}
        
        if("ETag" not in res.headers):
            print("Error in uploading file to S3: ETag not found in response header")
            print(res.text)
            return {"isSuccess": False}
        
        return {
            "isSuccess": True,
            "e_tag": res.headers["ETag"][1:-1],
            "part_id": str(chunk_id)
        }
    


from layernext.dataset.dataset import Dataset
from .sync import DatasetSync
from .datasetinterface import DatasetInterface


class DatasetClient:
    """
    Python SDK of Dataset download
    """
    def __init__(self, encoded_key_secret: str, layernext_url: str, download_path: str = None) -> None:
        _datalake_url = f'{layernext_url}/dataset' #/dataset :4000
        self.dataset_interface = DatasetInterface(encoded_key_secret, _datalake_url)
        self._dataset_sync_tool = DatasetSync(encoded_key_secret, layernext_url, download_path)

    """
    Download dataset
    @param version_id - id of dataset version 
    @param export_type - dataset export format """
    def download_dataset(self, version_id: str, export_type: str):
        self._dataset_sync_tool.download_dataset(version_id, export_type)

    """
    Download collection annotations
    From datalake
    @param collection_id - id of collection
    @param model_id - Optional: id of the model (same operation_id given in upload annotations) 
    if we need annotations for that specific model """
    def download_annotations(self, collection_id: str, annotation_type, operation_id_list):
        self._dataset_sync_tool.download_collection(collection_id, annotation_type, operation_id_list)

    
    """
    Download project annotations
    From datalake
    @param project_id - id of collection
    if we need annotations for that specific model """
    def download_annotations_for_project_v2(self, project_id_list, status_list, is_annotated_only):
        self._dataset_sync_tool.download_project_v2(project_id_list, status_list, is_annotated_only)


    
    """
    create dataset from search objects
    """
    def create_dataset(
            self, 
            dataset_name, 
            selection_id, 
            split_info, 
            labels, 
            export_types, 
            operation_list, 
            augmentation_list):
        _dataset = Dataset(client=self)
        response = _dataset.create_dataset(
            dataset_name, 
            selection_id, 
            split_info, 
            labels, 
            export_types, 
            operation_list,
            augmentation_list)
        return response

    """
    update dataset from search objects
    """
    def update_dataset_version(
            self, 
            version_id, 
            selection_id, 
            split_info, 
            labels, 
            export_types, 
            is_new_version_required, 
            operation_list,
            augmentation_list):
        _dataset = Dataset(client=self)
        response = _dataset.update_dataset_version(
            version_id, 
            selection_id, 
            split_info, 
            labels, 
            export_types, 
            is_new_version_required, 
            operation_list,
            augmentation_list)
        return response


    """
    delete dataset version
    """
    def delete_dataset_version(self, version_id):
        _dataset = Dataset(client=self)
        response = _dataset.delete_dataset_version(version_id)
        return response



import requests
from layernext.datalake.storage_client import StorageClient


class GCSClient(StorageClient):
    def upload_chunk(self, url, chunk_id, chunk_data, content_type, content_range, read_bytes):

        if(read_bytes == 0):
            return {
            "isSuccess": True,
            "e_tag": "",
            "part_id": str(chunk_id)
        }

        headers={"Content-Type": content_type, "Content-Range": content_range, "Content-Length": f'{read_bytes}'}

        res = requests.put(url, data=chunk_data, headers=headers)
        if(res.status_code != 200 and res.status_code != 201 and res.status_code != 308):
            print("Error in uploading file to S3: Status code: " + str(res.status_code))
            print(res.text)
            return {"isSuccess": False}

        return {
            "isSuccess": True,
            "e_tag": "",
            "part_id": str(chunk_id)
        }

    


from typing import TYPE_CHECKING
from .logger import get_debug_logger

if TYPE_CHECKING:
    from . import DatasetClient



annotation_logger = get_debug_logger('Dataset')


class Dataset:
    def __init__(self, client: "DatasetClient"):
        self._client = client


    """
    create dataset from search objects
    """
    def create_dataset(self, dataset_name, selection_id, split_info, labels, export_types, operation_list, augmentation_list):
        if(dataset_name == "" or dataset_name == None):
            return {
                "isSuccess": False,
                "message": "dataset name not valid"
            }
        if(selection_id == "" or selection_id == None):
            return {
                "isSuccess": False,
                "message": "selection_id not valid"
            }
        validationObj = self.validate_split_info(split_info)
        if(validationObj["isValidate"] == False):
            return {
                "isSuccess": False,
                "message": validationObj["message"]
            }
        if augmentation_list != None:
            if "IMAGE_LEVEL" in augmentation_list:
                for i in range(len(augmentation_list["IMAGE_LEVEL"])):
                    augmentation_list["IMAGE_LEVEL"][i]["isSelected"] = True

        split_info = {
            "train": split_info["train"],
            "test": split_info["test"],
            "validation": split_info["validation"]
        }
        payload_dataset_creation = {
            "name": dataset_name,
            "splitInfo": split_info,
            "labels": labels,
            "exportTypes": export_types,
            "operations": operation_list,
            "augmentations": augmentation_list
        }

        response = self._client.dataset_interface.create_dataset(selection_id, payload_dataset_creation)
        
        return response


    """
    update dataset from search objects
    """
    def update_dataset_version(
            self, 
            version_id, 
            selection_id, 
            split_info, 
            labels, 
            export_types, 
            is_new_version_required, 
            operation_list,
            augmentation_list):
        if(version_id == "" or version_id == None):
            return {
                "isSuccess": False,
                "message": "dataset name not valid"
            }
        if(selection_id == "" or selection_id == None):
            return {
                "isSuccess": False,
                "message": "selection_id not valid"
            }     
        validationObj = self.validate_split_info(split_info)
        if(validationObj["isValidate"] == False):
            return {
                "isSuccess": False,
                "message": validationObj["message"]
            }
        
        if augmentation_list != None:
            if "IMAGE_LEVEL" in augmentation_list:
                for i in range(len(augmentation_list["IMAGE_LEVEL"])):
                    augmentation_list["IMAGE_LEVEL"][i]["isSelected"] = True

        split_info = {
            "train": split_info["train"],
            "test": split_info["test"],
            "validation": split_info["validation"]
        }
        payload_dataset_update = {
            "splitInfo": split_info,
            "labels": labels,
            "exportTypes": export_types,
            "operations": operation_list,
            "augmentations": augmentation_list
        }
        if(is_new_version_required == False):
            response = self._client.dataset_interface.update_existing_dataset_version(selection_id, payload_dataset_update, version_id)
        else:
            response = self._client.dataset_interface.create_new_dataset_version(selection_id, payload_dataset_update, version_id)
        return response
    
    """
    validate split info object for dataset creation and update
    """
    def validate_split_info(self, split_info):

        if ("train" not in split_info) or ("test" not in split_info) or ("validation" not in split_info):
            return {
                "isValidate": False,
                "message": "split info not valid"
            }
        elif (split_info["train"] + split_info["test"] + split_info["validation"]) != 100:
            return {
                "isValidate": False,
                "message": "Invalid split percentages. The test, train, and validation splits must sum up to 100%."
            }
        else:
            return {
                "isValidate": True,
                "message": "split info valid"
            }
    
    
    """
    delete dataset version
    """
    def delete_dataset_version(self, version_id):
        response = self._client.dataset_interface.delete_dataset_version(version_id)
        
        return response
    


import requests
from layernext.datalake.storage_client import StorageClient
from azure.storage.blob import BlobClient
import base64

class AzureClient(StorageClient):
    def __init__(self, url_list) -> None:
        super().__init__()

        # create a blob client for the file, take first url out of list as all urls are same in azure
        first_url_obj = url_list[0]
        self.block_blob_client = BlobClient.from_blob_url(first_url_obj["signedUrl"])

    def upload_chunk(self, url, chunk_id, chunk_data, content_type, content_range, read_bytes):

        # upload the block
        # block_id =f"{chunk_id:06d}"
        block_id = base64.b64encode(f"{chunk_id:06}".encode()).decode()

        try:
            self.block_blob_client.stage_block(block_id, chunk_data)
        except Exception as e:
            print("Exception in AzureClient during stage_block:", e)

        return {
            "isSuccess": True,
            "e_tag": "",
            "part_id": block_id
        }


    


import json
from multiprocessing.pool import ThreadPool
import os
from typing import TYPE_CHECKING
from .logger import get_debug_logger

if TYPE_CHECKING:
    from . import StudioClient



annotation_logger = get_debug_logger('Project')


class Project:
    def __init__(self, client: "StudioClient"):
        self._client = client


    """
    create the project
    """
    def create_project(self, project_name, selection_id, fps, frames_per_task, assign_to_all,send_email, default_shape_type):
        
        # check for validity of default_shape_type
        if default_shape_type != "rectangle" and default_shape_type != "polygon" and default_shape_type != "line":
            default_shape_type = "rectangle"
        payload_project_creation = {
            "selectionId": selection_id,
            "name": project_name,
            "fps": fps,
            "framesPerTask": frames_per_task,
            "assignToAll": assign_to_all,
            "sendEmail" : send_email,
            "defaultShapeType": default_shape_type
        }
        # print(payload_initial_project_creation)
        response = self._client.studio_interface.create_initial_project()
        get_selection_id_success = True
        if("isSuccess" in response):
            if(response["isSuccess"] == False):
                get_selection_id_success = False
        if get_selection_id_success == True:
            print('project id: ',response["id"])
            project_response = self._client.studio_interface.create_or_update_project(response["id"], payload_project_creation, False)
            return project_response
        else:
            print({"isSuccess": False})
            return {"isSuccess": False}
    

    """
    update the project
    """
    def update_project(self, project_id, selection_id, fps, frames_per_task, assign_to_all,send_email, default_shape_type):
        
        # check for validity of default_shape_type
        if default_shape_type != "rectangle" and default_shape_type != "polygon" and default_shape_type != "line":
            default_shape_type = "rectangle"
        payload_project_creation = {
            "selectionId": selection_id,
            "name": None,
            "fps": fps,
            "framesPerTask": frames_per_task,
            "assignToAll": assign_to_all,
            "sendEmail": send_email,
            "defaultShapeType": default_shape_type
        }
        
        response  = self._client.studio_interface.create_or_update_project(project_id, payload_project_creation, True)
        
        return response

    """
    delete the project
    """
    def delete_project(self, project_id):
      response  = self._client.studio_interface.delete_project(project_id)

      return response


    """
    update the labels of project
    """
    def update_labels_to_project(self, project_id, add_list, remove_list):
      response  = self._client.studio_interface.update_labels_to_project(project_id, add_list, remove_list)
      return response

    """
    update the labels of project from label group
    """
    def update_labels_from_group_to_project(self, project_id, group_id):
      response  = self._client.studio_interface.set_project_label_group(project_id, group_id)
      return response

    """
    Get list of studio project
    """
    def get_project_list(self):
        response = self._client.studio_interface.list_projects()
        return response

    

    


import traceback
import requests


class StudioInterface:

    def __init__(self, auth_token: str, studio_url: str):
        self.auth_token = auth_token
        self.studio_url = studio_url

    ''''
    get selection id from query, filter and collectionId
    '''
    def create_initial_project(self):
        hed = {'Authorization': 'Basic ' + self.auth_token}
        url = f'{self.studio_url}/api/client/projects/initial/create'

        try:
            response = requests.get(url=url, json={}, headers=hed)
            status_code = response.status_code
            if status_code == 200:
                return response.json()
            else:
                return {"isSuccess": False}
        #Handle connection error
        except requests.exceptions.ConnectionError as e:
            print("Connection error from Data Lake connection")
            return {"isSuccess": False, "message": "Failed to connect to Data Lake"}
        #Handle timeout error
        except requests.exceptions.Timeout as e:
            print("Timeout error from Data Lake connection")
            return {"isSuccess": False, "message": "Failed to connect to Data Lake"}
        #Handle HTTP errors
        except requests.exceptions.HTTPError as e:
            print("HTTP error from Data Lake connection")
            return {"isSuccess": False, "message": "Failed to connect to Data Lake"} 
        except requests.exceptions.RequestException as e:
            print(f"An unexpected request exception occurred: {format(e)}")
            traceback.print_exc()
            return {"isSuccess": False, "message": "Error in checking version compatibility"}
        except Exception as e1:
            print(f"An unexpected exception occurred: {format(e1)}")
            traceback.print_exc()
            return {"isSuccess": False, "message": "Error in checking version compatibility"}

    '''
    update of create project
    '''
    def create_or_update_project(self, project_id, payload, isEditable):
        hed = {'Authorization': 'Basic ' + self.auth_token}
        url = f'{self.studio_url}/api/client/project/{project_id}/update?isEditable={isEditable}'

        try:
            response = requests.post(url=url, json=payload, headers=hed)
            status_code = response.status_code
            if status_code == 200:
                return response.json()
            else:
                return {"isSuccess": False, "message": response.text}
        #Handle connection error
        except requests.exceptions.ConnectionError as e:
            print("Connection error from Data Lake connection")
            return {"isSuccess": False, "message": "Failed to connect to Data Lake"}
        #Handle timeout error
        except requests.exceptions.Timeout as e:
            print("Timeout error from Data Lake connection")
            return {"isSuccess": False, "message": "Failed to connect to Data Lake"}
        #Handle HTTP errors
        except requests.exceptions.HTTPError as e:
            print("HTTP error from Data Lake connection")
            return {"isSuccess": False, "message": "Failed to connect to Data Lake"} 
        except requests.exceptions.RequestException as e:
            print(f"An unexpected request exception occurred: {format(e)}")
            traceback.print_exc()
            return {"isSuccess": False, "message": "Error in checking version compatibility"}
        except Exception as e1:
            print(f"An unexpected exception occurred: {format(e1)}")
            traceback.print_exc()
            return {"isSuccess": False, "message": "Error in checking version compatibility"}

    
    '''
    update of create project
    '''
    def delete_project(self, project_id):
        hed = {'Authorization': 'Basic ' + self.auth_token}
        url = f'{self.studio_url}/api/client/project/{project_id}/delete'

        try:
            response = requests.post(url=url, json={}, headers=hed)
            status_code = response.status_code
            if status_code == 200:
                return response.json()
            else:
                return {"isSuccess": False, "errorMsg": response.text}
        #Handle connection error
        except requests.exceptions.ConnectionError as e:
            print("Connection error from Data Lake connection")
            return {"isSuccess": False, "errorMsg": "Failed to connect to Data Lake"}
        #Handle timeout error
        except requests.exceptions.Timeout as e:
            print("Timeout error from Data Lake connection")
            return {"isSuccess": False, "errorMsg": "Failed to connect to Data Lake"}
        #Handle HTTP errors
        except requests.exceptions.HTTPError as e:
            print("HTTP error from Data Lake connection")
            return {"isSuccess": False, "errorMsg": "Failed to connect to Data Lake"} 
        except requests.exceptions.RequestException as e:
            print(f"An unexpected request exception occurred: {format(e)}")
            traceback.print_exc()
            return {"isSuccess": False, "errorMsg": "Error in checking version compatibility"}
        except Exception as e1:
            print(f"An unexpected exception occurred: {format(e1)}")
            traceback.print_exc()
            return {"isSuccess": False, "errorMsg": "Error in checking version compatibility"}
    

    """
    update the labels of project
    """
    def update_labels_to_project(self, project_id, add_list, remove_list):
        payload = {
            "addList": add_list,
            "removeList": remove_list
        }

        hed = {'Authorization': 'Basic ' + self.auth_token}
        url = f'{self.studio_url}/api/client/project/{project_id}/updateLabel'

        try:
            response = requests.post(url=url, json=payload, headers=hed)
            status_code = response.status_code
            if status_code == 200:
                return response.json()
            else:
                return {"isSuccess": False}
        #Handle connection error
        except requests.exceptions.ConnectionError as e:
            print("Connection error from Data Lake connection")
            return {"isSuccess": False, "message": "Failed to connect to Data Lake"}
        #Handle timeout error
        except requests.exceptions.Timeout as e:
            print("Timeout error from Data Lake connection")
            return {"isSuccess": False, "message": "Failed to connect to Data Lake"}
        #Handle HTTP errors
        except requests.exceptions.HTTPError as e:
            print("HTTP error from Data Lake connection")
            return {"isSuccess": False, "message": "Failed to connect to Data Lake"} 
        except requests.exceptions.RequestException as e:
            print(f"An unexpected request exception occurred: {format(e)}")
            traceback.print_exc()
            return {"isSuccess": False, "message": "Error in checking version compatibility"}
        except Exception as e1:
            print(f"An unexpected exception occurred: {format(e1)}")
            traceback.print_exc()
            return {"isSuccess": False, "message": "Error in checking version compatibility"}


    """
    Get list of studio project
    """
    def list_projects(self):
        hed = {'Authorization': 'Basic ' + self.auth_token}
        url = f'{self.studio_url}/api/client/projects/list'

        try:
            response = requests.get(url=url, headers=hed)
            return response.json()
        #Handle connection error
        except requests.exceptions.ConnectionError as e:
            print("Connection error from Data Lake connection")
        #Handle timeout error
        except requests.exceptions.Timeout as e:
            print("Timeout error from Data Lake connection")
        #Handle HTTP errors
        except requests.exceptions.HTTPError as e:
            print("HTTP error from Data Lake connection")
        except requests.exceptions.RequestException as e:
            print(f"An unexpected request exception occurred: {format(e)}")
            traceback.print_exc()
        except Exception as e1:
            print(f"An unexpected exception occurred: {format(e1)}")
            traceback.print_exc()

    """
    Update the label group for the project
    """
    def set_project_label_group(self, project_id, group_id):
        hed = {'Authorization': 'Basic ' + self.auth_token}
        url = f'{self.studio_url}/api/client/project/{project_id}/setLabelGroup'
        payload = {
            "labelGroupId": group_id,
        }
        try:
            response = requests.post(url=url, json=payload, headers=hed)
            status_code = response.status_code
            if status_code == 200:
                return response.json()
            else:
                return None
        #Handle connection error
        except requests.exceptions.ConnectionError as e:
            print("Connection error from Data Lake connection")
            return None
        #Handle timeout error
        except requests.exceptions.Timeout as e:
            print("Timeout error from Data Lake connection")
            return None
        #Handle HTTP errors
        except requests.exceptions.HTTPError as e:
            print("HTTP error from Data Lake connection")
            return None
        except requests.exceptions.RequestException as e:
            print(f"An unexpected request exception occurred: {format(e)}")
            traceback.print_exc()
            return None
        except Exception as e1:
            print(f"An unexpected exception occurred: {format(e1)}")
            traceback.print_exc()
            return None




from layernext.studio.project import Project
from .studiointerface import StudioInterface
from .logger import get_debug_logger

# studio_logger = get_debug_logger('StudioClient')


class StudioClient:
    """
    Python SDK of Datalake
    """

    def __init__(self, encoded_key_secret: str, layernext_url: str) -> None:
        _studio_url = f'{layernext_url}/studio' #/studio  :8080

        self.studio_interface = StudioInterface(encoded_key_secret, _studio_url)
    
    def create_project(self, project_name, selection_id, fps, frames_per_task, assign_to_all, send_email, default_shape_type):
        if project_name == '':
            print('Project name is empty')
            return None
        #print('project details: ', project_name, selection_id)
        _project = Project(client=self)
        response = _project.create_project(project_name, selection_id, fps, frames_per_task, assign_to_all, send_email, default_shape_type)
        return response
    
    """
    update the project
    """
    def update_project(self, project_id, selection_id, fps, frames_per_task, assign_to_all, send_email, default_shape_type):
        #print('project details: ', project_id, selection_id)
        _project = Project(client=self)
        response = _project.update_project(project_id, selection_id, fps, frames_per_task, assign_to_all, send_email, default_shape_type)
        return response

    """
    delete the project
    """
    def delete_project(self, project_id):
        #print('project details: ', project_id)
        _project = Project(client=self)
        response = _project.delete_project(project_id)
        return response

    """
    update the labels of project
    """
    def update_labels_to_project(self, project_id, add_list, remove_list):
        if project_id == '':
            print('Project Id is empty')
            return None
        _project = Project(client=self)
        response = _project.update_labels_to_project(project_id, add_list, remove_list)
        return response

    """
    Get list of studio project
    """
    def studio_project_list(self):
        _project = Project(client=self)
        response = _project.get_project_list()
        return response

    """
    Set label group
    """
    def project_set_label_group(self, project_id, group_id):
        if project_id == '' or group_id == '':
            print('Project Id or Group Id is empty')
            return None
        _project = Project(client=self)
        return _project.update_labels_from_group_to_project(project_id, group_id)







from typing import TYPE_CHECKING

from .annotation import Annotation
from .constants import OPERATION_MODE_AUTO, AnnotationUploadType
from .logger import get_debug_logger


if TYPE_CHECKING:
    from . import DatalakeClient

model_logger = get_debug_logger('Model')


class ModelRun(Annotation):
    def __init__(self, client: "DatalakeClient"):
        super().__init__(client)

    def upload_modelrun_json(self, storage_base_path: str, model_id: str, file_path: str, annotation_geometry: str, is_normalized: bool, dest_project_id: str, version: str, bucket_name: str, upload_type: AnnotationUploadType):
        model_logger.debug(f'Started uploading model run data for model_id: {model_id}')
        self.upload_annotation_json(storage_base_path, model_id, file_path, annotation_geometry, OPERATION_MODE_AUTO, is_normalized, dest_project_id, version, bucket_name, upload_type)
        model_logger.debug(f'Finished uploading model run data for model_id: {model_id}')




import base64

from layernext import datalake, dataset, studio
from layernext.datalake.constants import AnnotationUploadType, MediaType, ObjectType

__version__ = '1.1.0b1'


class LayerNextClient:
    """
    Python SDK of LayerNext
    """

    def __init__(self, api_key: str, secret: str, layernext_url: str) -> None:
        _string_key_secret = f'{api_key}:{secret}'
        _key_secret_bytes = _string_key_secret.encode("ascii")
        _encoded_key_secret_bytes = base64.b64encode(_key_secret_bytes)
        self.encoded_key_secret = _encoded_key_secret_bytes.decode("ascii")
        self.layernext_url = layernext_url
        self.__check_sdk_version_compatibility()

    def __check_sdk_version_compatibility(self):
        """
        Check if the SDK version is compatible with the datalake backend version
        """
        _datalake_client = datalake.DatalakeClient(
            self.encoded_key_secret, self.layernext_url)

        version_id = __version__.split('b')[0]

        _datalake_client.check_sdk_version_compatibility(version_id)

    """
    Annotation data upload from json file with new format and unique_name
    """
    def upload_annotations_by_unique_name    (
            self,
            operation_unique_id: str,
            json_data_file_path: str,
            is_normalized: bool,
            is_model_run: bool
    ):
        
        self.__upload_annotations(
            None,
            operation_unique_id,
            json_data_file_path,
            None,
            is_normalized,
            is_model_run,
            upload_type=AnnotationUploadType.BY_UNIQUE_NAME
        )

    """
    Annotation data upload from json file with new format and bucket name
    """
    def upload_annotations_by_storage_path    (
            self,
            operation_unique_id: str,
            json_data_file_path: str,
            is_normalized: bool,
            is_model_run: bool,
            bucket_name: str = None
    ):
        
        self.__upload_annotations(
            None,
            operation_unique_id,
            json_data_file_path,
            None,
            is_normalized,
            is_model_run,
            bucket_name=bucket_name,
            upload_type=AnnotationUploadType.BY_STORAGE_PATH
        )

    """
    Old annotation data upload from json file
    """
    def upload_annoations_for_folder    (
            self,
            collection_base_path: str,
            operation_unique_id: str,
            json_data_file_path: str,
            shape_type: str,
            is_normalized: bool,
            is_model_run: bool,
            destination_project_id: str = None
    ):
        
        self.__upload_annotations(
            collection_base_path,
            operation_unique_id,
            json_data_file_path,
            shape_type,
            is_normalized,
            is_model_run,
            destination_project_id,
            None,
            upload_type=AnnotationUploadType.BY_FILE_NAME_OR_UNIQUE_NAME
        )

    """
    New annotation data upload from json file with new format and file name
    """
    def upload_annotations_for_folder    (
            self,
            collection_name: str,
            operation_unique_id: str,
            json_data_file_path: str,
            is_normalized: bool,
            is_model_run: bool
    ):
        
        self.__upload_annotations(
            collection_name,
            operation_unique_id,
            json_data_file_path,
            None,
            is_normalized,
            is_model_run,
            upload_type=AnnotationUploadType.BY_FILE_NAME
        )
        

    def __upload_annotations(
            self,
            collection_base_path: str,
            operation_unique_id: str,
            json_data_file_path: str,
            shape_type: str,
            is_normalized: bool,
            is_model_run: bool,
            destination_project_id: str = None,
            version: str = "v2",
            bucket_name: str = None,
            upload_type: AnnotationUploadType = 1
    ):
        """
        Upload annotation data from a json file
        """
        # init datalake client
        _datalake_client = datalake.DatalakeClient(
            self.encoded_key_secret, self.layernext_url)

        if is_model_run:
            _datalake_client.upload_modelrun_from_json(
                collection_base_path,
                operation_unique_id,
                json_data_file_path,
                shape_type,
                is_normalized,
                destination_project_id,
                version,
                bucket_name,
                upload_type
            )
        else:
            _datalake_client.upload_groundtruth_from_json(
                collection_base_path,
                operation_unique_id,
                json_data_file_path,
                shape_type,
                is_normalized,
                destination_project_id,
                version,
                bucket_name,
                upload_type
            )

    """
    Download dataset
    """

    def download_dataset(self, version_id: str, export_type: str, custom_download_path: str = None):
        # init dataset client
        _dataset_client = dataset.DatasetClient(
            self.encoded_key_secret, self.layernext_url, custom_download_path)

        # start download
        _dataset_client.download_dataset(version_id, export_type)

    """"
    Images/video upload - deprecated
    """

    def file_upload(self, path: str, collection_type, collection_name, meta_data_object="", override=False):
        _datalake_client = datalake.DatalakeClient(
            self.encoded_key_secret, self.layernext_url)
        return _datalake_client.file_upload(path, collection_type, collection_name, meta_data_object, override)

    """"
    Images/video upload to a collection
    @param path: local folder path (absolute)
    @param content_type: 'image' or 'video'
    @param collection_name: Collection which files to be uploaded to.
    If existing one is given new files are added to that collection
    @param meta_data_object: Object to specify custom meta data fields and flags
    @param meta_data_override: override provided meta_data_object in case of the file exist already in datalake
    """

    def upload_files_to_collection(
            self,
            path: str,
            content_type: str,
            collection_name: str,
            meta_data_object={},
            meta_data_override=False,
            json_file_path=None,
            storage_prefix_path=None,
            application_code=None
    ):
        _datalake_client = datalake.DatalakeClient(
            self.encoded_key_secret, self.layernext_url)
        if collection_name == "":
            print(f'Invalid collection name "{collection_name}"')
            return {
                "is_success": False,
                "collection_id": None
            }
        if content_type.lower() == "image":
            collection_type = MediaType.IMAGE.value
        elif content_type.lower() == "video":
            collection_type = MediaType.VIDEO.value
        else:
            collection_type = MediaType.OTHER.value

        upload_details = _datalake_client.file_upload(
            path,
            collection_type,
            collection_name,
            meta_data_object,
            application_code,
            meta_data_override,
            storage_prefix_path
        )

        # if json file path is provided and upload is successful, upload metadata for the files
        if json_file_path is not None and upload_details["is_success"] == True:
            job_id = upload_details["job_id"]
            self.wait_for_job_complete(job_id)
            self.upload_metadata_for_files(collection_name, json_file_path)

        return upload_details

    """"
    trash common logic for sdk
    @param collection_id: collection id
    @param query: query to filter the objects
    @param filter: filter to filter the objects
    @param object_type: 'image' or 'video'
    @param object_list: list of object ids
    @param is_all_selected: boolean
    """

    def __trash_objects(
        self,
        collection_id: str,
        query: str,
        filter,
        object_type: int,
        object_list,
        is_all_selected=True
    ):

        filter = self.__create_filter(filter)

        _datalake_client = datalake.DatalakeClient(
            self.encoded_key_secret, self.layernext_url)

        response = _datalake_client.get_selection_id(
            collection_id, query, filter, object_type, object_list, is_all_selected)

        print('selection id: ', response)
        get_selection_id_success = True
        if ("isSuccess" in response):
            if (response["isSuccess"] == False):
                get_selection_id_success = False
        if (get_selection_id_success):
            selection_id = response['selectionTag']
            return _datalake_client.trash_datalake_object(selection_id)
        else:
            print('trash_response: ', {
                "isSuccess": False
            })
            return {
                "isSuccess": False,
                "message": response["warningMsg"]
            }

    """"
    Images/video trash from datalake
    @param datalake_query: query to select objects
    @param datalake_filter: filter to select objects
    @param content_type: 'image' or 'video'
    """

    def trash_objects_from_datalake(
        self,
        datalake_query: str,
        datalake_filter={},
        content_type: str = "image",
    ):
        # print(f'create project using query, filter and collections - project name: {project_name}, ')

        # Prevent all objects case
        if datalake_query == "" and (datalake_filter == "" or datalake_filter == {}):
            print('At least either valid filter or query should be given')
            return {
                "is_success": False,
                "error": "Not enough selection parameters"
            }

        if content_type == "image":
            object_type = ObjectType.IMAGE.value
        elif content_type == "video":
            object_type = ObjectType.VIDEO.value
        else:
            print('No valid content type')
            return {
                "is_success": False,
                "error": "Should specify either image or video as content type"
            }
        try:

            response = self.__trash_objects(
                "", datalake_query, datalake_filter, object_type, [])
            return response
        except Exception as e:
            print("An exception occurred", format(e))
            return {
                "is_success": False,
                "error": f'An exception occurred: {format(e)}'
            }

    """"
    Images/video trash from collection
    @param collection id: collection id
    @param query: query to select objects
    @param filter: filter to select objects
    """

    def trash_objects_from_collection(
        self,
        collection_id: str,
        query: str = "",
        filter={},
    ):

        if (filter == None):
            filter = {}

        if (collection_id == None or collection_id == ''):
            print('collection id must be provided')
            return {
                "isSuccess": False,
                "error": 'collection id must be provided'
            }
        object_type_object = self.__get_object_type_by_id(collection_id)
        object_type = None
        if "objectType" in object_type_object:
            object_type = object_type_object["objectType"]

        if object_type == None:
            print('object type cannot find for collection id')
            return {
                "isSuccess": False,
                "error": 'object type cannot find for collection id'
            }

        try:
            # if query and filter are empty, collection will be trashed
            if (query == "" or query == None) and (filter == "" or filter == {}):
                response = self.__trash_objects(
                    "", query, filter, object_type, [collection_id], False)
            else:
                response = self.__trash_objects(
                    collection_id, query, filter, 0, [], True)

            print('trash_response: ', response.get("message"))
            return response
        except Exception as e:
            print("An exception occurred", format(e))
            return {
                "isSuccess": False,
                "error": f'An exception occurred: {format(e)}'
            }

    """
    Deprecated
    Download collection annotations from datalake
    @param collection_id - id of collection
    @param operation_id - Optional: id of the model (same operation_id given in upload annotations) - default: None
    @param custom_download_path - Optional: custom download path for save downloaded files - default: None
    """

    def download_annotations(self, collection_id: str, operation_id, custom_download_path: str = None):

        print('This method is now deprecated and please switch to "download_collection" function as this will be removed in future')
        # init dataset client
        _dataset_client = dataset.DatasetClient(
            self.encoded_key_secret, self.layernext_url, custom_download_path)

        operation_id_list = []
        annotation_type = "all"
        if operation_id != None:
            operation_id_list.append(operation_id)
        else:
            annotation_type = "human"

        # start download
        _dataset_client.download_annotations(
            collection_id, annotation_type, operation_id_list)

    """
    Download collection annotations from datalake
    @param collection_id - id of collection
    @param annotation_type - Optional: annotation category, can be given ("machine", "human" or "all") - default: "all"
    @param operation_id_list - Optional: id list of the model or ground truth (same operation_id_list given in upload annotations) - default: []
    @param custom_download_path - Optional: custom download path for save downloaded files - default: None
    """

    def download_collection(self, collection_id: str, annotation_type="all", operation_id_list=[], custom_download_path: str = None):
        # init dataset client
        _dataset_client = dataset.DatasetClient(
            self.encoded_key_secret, self.layernext_url, custom_download_path)

        # start download
        _dataset_client.download_annotations(
            collection_id, annotation_type, operation_id_list)

    """"
    Images and annotation upload
    """

    def upload_data(self, collection_name, file_path, meta_data_object, operation_unique_id, json_data_file_path, shape_type, is_normalized, is_model_run):

        upload_details = None
        if (file_path == None):
            print('file upload cannot be done')
        else:
            upload_details = self.file_upload(
                file_path, MediaType.IMAGE.value, collection_name, meta_data_object, False)

        if (json_data_file_path == None):
            print('annotation upload cannot be done')
        else:
            job_id = upload_details["job_id"]
            self.wait_for_job_complete(job_id)
            self.__upload_annotations(
                collection_name, operation_unique_id, json_data_file_path, shape_type, is_normalized, is_model_run)

        return upload_details

    """"
    get upload progress
    """

    def get_upload_status(self, collection_name):
        _datalake_client = datalake.DatalakeClient(
            self.encoded_key_secret, self.layernext_url)
        return _datalake_client.get_upload_status(collection_name)

    """
    remove annotations of collection model run
    """

    def remove_annotations(self, collection_id: str, model_run_id: str):
        print(
            f'remove annotations of collection: {collection_id}, operation id: {model_run_id}')

        _datalake_client = datalake.DatalakeClient(
            self.encoded_key_secret, self.layernext_url)

        _datalake_client.remove_collection_annotations(
            collection_id, model_run_id)

    """
    use to create appropriate filter object from user filter input
    """

    def __create_filter(self, filter):
        # { "annotation_types": ["human", "raw", "machine"], "from_date": "", "to_date": "" }
        return_filter = {
            "filterBy": [],
            "date": {}
        }
        if "annotation_types" in filter:
            if "human" in filter["annotation_types"]:
                return_filter["filterBy"].append(2)
            if "raw" in filter["annotation_types"]:
                return_filter["filterBy"].append(0)
            if "machine" in filter["annotation_types"]:
                return_filter["filterBy"].append(1)
        if "to_date" in filter:
            return_filter["date"]["toDate"] = filter["to_date"]
        if "from_date" in filter:
            return_filter["date"]["fromDate"] = filter["from_date"]
        return return_filter

    """
    create annotation studio project using query, filter and collections
    """

    def __create_studio_project(
        self,
        project_name: str,
        collection_id: str,
        query: str,
        filter,
        object_type: int,
        object_list,
        fps,
        frames_per_task,
        assign_to_all,
        send_email,
        default_shape_type
    ):
        # print(f'create project using query, filter and collections - project name: {project_name}, ')
        filter = self.__create_filter(filter)

        _datalake_client = datalake.DatalakeClient(
            self.encoded_key_secret, self.layernext_url)
        _studio_client = studio.StudioClient(
            self.encoded_key_secret, self.layernext_url)

        response = _datalake_client.get_selection_id(
            collection_id, query, filter, object_type, object_list)
        print('selection id: ', response)
        get_selection_id_success = True
        if ("isSuccess" in response):
            if (response["isSuccess"] == False):
                get_selection_id_success = False
        if get_selection_id_success == True:
            project_response = _studio_client.create_project(
                project_name,
                response['selectionTag'],
                fps,
                frames_per_task,
                assign_to_all,
                send_email,
                default_shape_type)
            return project_response
        else:
            print('project_response: ', {
                "isSuccess": False
            })
            return {
                "isSuccess": False
            }

    def __get_object_type_by_id(self, object_id: str):
        print(f'get object type by id: {object_id}')

        _datalake_client = datalake.DatalakeClient(
            self.encoded_key_secret, self.layernext_url)
        response = _datalake_client.get_object_type_by_id(object_id)
        return response

    """
    create annotation studio project using query, filter and collections
    """

    def create_annotation_project_from_collection(
        self,
        project_name: str,
        collection_id: str,
        query: str = "",
        filter={},
        fps: int = 4,
        frames_per_task: int = 120,
        assign_to_all=False,
        send_email=False,
        default_shape_type="rectangle"
    ):
        # print(f'create project using query, filter and collections - project name: {project_name}, ')

        if (collection_id == None or collection_id == ''):
            print('collection id must be provided')
            return {
                "isSuccess": False,
                "error": 'collection id must be provided'
            }
        object_type_object = self.__get_object_type_by_id(collection_id)
        object_type = None
        if "objectType" in object_type_object:
            object_type = object_type_object["objectType"]

        if object_type == None:
            print('object type cannot find for collection id')
            return {
                "isSuccess": False,
                "error": 'object type cannot find for collection id'
            }

        # if((fps == None) and (object_type == ObjectType.VIDEO.value or object_type == ObjectType.VIDEO_COLLECTION.value)):
        #    print('fps must be provided')
        #    return {
        #        "isSuccess": False,
        #        "error": 'fps must be provided'
        #    }

        try:
            response = self.__create_studio_project(
                project_name,
                collection_id,
                query,
                filter,
                0,
                [],
                fps,
                frames_per_task,
                assign_to_all,
                send_email,
                default_shape_type)
            return response
        except Exception as e:
            print("An exception occurred", format(e))
            return {
                "isSuccess": False,
                "error": f'An exception occurred: {format(e)}'
            }

    """
    create annotation studio project using query, filter and collections
    """

    def create_annotation_project_from_datalake(
        self,
        project_name: str,
        datalake_query: str,
        datalake_filter={},
        content_type: str = "image",
        fps: int = 4,
        frames_per_task: int = 120,
        assign_to_all=False,
        send_email=False,
        default_shape_type="rectangle"
    ):
        # print(f'create project using query, filter and collections - project name: {project_name}, ')

        # Prevent all objects case
        if datalake_query == "" and (datalake_filter == "" or datalake_filter == {}):
            print('At least either valid filter or query should be given')
            return {
                "is_success": False,
                "error": "Not enough selection parameters"
            }

        # if(object_type == ObjectType.VIDEO or object_type == ObjectType.VIDEO_COLLECTION):
        #    return {
        #        "isSuccess": False,
        #        "error": 'fps must be provided'
        #    }
        if content_type == "image":
            object_type = ObjectType.IMAGE.value
        elif content_type == "video":
            object_type = ObjectType.VIDEO.value
        else:
            print('No valid content type')
            return {
                "is_success": False,
                "error": "Should specify either image or video as content type"
            }
        try:
            response = self.__create_studio_project(
                project_name,
                "",
                datalake_query,
                datalake_filter,
                object_type,
                [],
                fps,
                frames_per_task,
                assign_to_all,
                send_email,
                default_shape_type)
            return response
        except Exception as e:
            print("An exception occurred", format(e))
            return {
                "is_success": False,
                "error": f'An exception occurred: {format(e)}'
            }

    """
    update annotation project using query, filter and collections
    """

    def __update_objects_to_studio_project(
        self,
        project_id: str,
        collection_id: str,
        query: str,
        filter,
        object_type: int,
        object_list,
        fps,
        frames_per_task,
        assign_to_all,
        send_email,
        default_shape_type="rectangle"
    ):
        # print(f'create project using query, filter and collections - project id: {project_id}, ')
        filter = self.__create_filter(filter)

        _datalake_client = datalake.DatalakeClient(
            self.encoded_key_secret, self.layernext_url)
        _studio_client = studio.StudioClient(
            self.encoded_key_secret, self.layernext_url)

        response = _datalake_client.get_selection_id(
            collection_id, query, filter, object_type, object_list)
        print('selection id: ', response)
        get_selection_id_success = True
        if ("isSuccess" in response):
            if (response["isSuccess"] == False):
                get_selection_id_success = False
        if get_selection_id_success == True:
            project_response = _studio_client.update_project(
                project_id,
                response['selectionTag'],
                fps,
                frames_per_task,
                assign_to_all,
                send_email,
                default_shape_type
            )
            print('project_response: ', project_response)
            return project_response
        else:
            print('project_response: ', {
                "is_success": False
            })
            return {
                "is_success": False
            }

    """
    update annotation studio project using query, filter and collections
    """

    def add_files_to_annotation_project_from_collection(
        self,
        project_id: str,
        collection_id: str,
        query: str,
        filter,
        fps: int = 0,
        frames_per_task: int = 120,
        assign_to_all=False,
        send_email=False,
        default_shape_type="rectangle"
    ):
        # print(f'create project using query, filter and collections - project name: {project_id}, ')

        if collection_id == None or collection_id == '':
            return {
                "is_success": False,
                "error": 'collection id must be provided'
            }

        object_type_object = self.__get_object_type_by_id(collection_id)
        object_type = None
        if "objectType" in object_type_object:
            object_type = object_type_object["objectType"]

        if object_type == None:
            print('object type cannot find for collection id')
            return {
                "is_success": False,
                "error": 'object type cannot find for collection id'
            }
        # if(fps == None and (object_type == ObjectType.VIDEO.value or object_type == ObjectType.VIDEO_COLLECTION.value)):
        #    return {
        #        "isSuccess": False,
        #        "error": 'fps must be provided'
        #    }

        try:
            response = self.__update_objects_to_studio_project(
                project_id,
                collection_id,
                query,
                filter,
                0,
                [],
                fps,
                frames_per_task,
                assign_to_all,
                send_email,
                default_shape_type)
            return response
        except Exception as e:
            print("An exception occurred", format(e))
            return {
                "is_success": False,
                "error": f'An exception occurred: {format(e)}'
            }

    """
    update annotation studio project using query, filter and collections
    """

    def add_files_to_annotation_project_from_datalake(
        self,
        project_id: str,
        query: str,
        filter,
        content_type: str,
        fps: int = 0,
        frames_per_task: int = 120,
        assign_to_all=False,
        send_email=False,
        default_shape_type="rectangle"
    ):
        # print(f'create project using query, filter and collections - project name: {project_id}, ')

        if query == "" and (filter == "" or filter == {}):
            print('At least either valid filter or query should be given')
            return {
                "is_success": False,
                "error": "Not enough selection parameters"
            }

        if content_type == "image":
            object_type = ObjectType.IMAGE.value
        elif content_type == "video":
            object_type = ObjectType.VIDEO.value
        else:
            print('No valid content type')
            return {
                "is_success": False,
                "error": "Should specify either image or video as content type"
            }
        try:
            response = self.__update_objects_to_studio_project(
                project_id,
                "",
                query,
                filter,
                object_type,
                [],
                fps,
                frames_per_task,
                assign_to_all,
                send_email,
                default_shape_type)
            return response
        except Exception as e:
            print("An exception occurred", format(e))
            return {
                "is_success": False,
                "error": f'An exception occurred: {format(e)}'
            }
    """
    delete a annotation studio project by project id
    """

    def delete_annotation_project(self, project_id: str):
        if (project_id == None or project_id == ""):
            print("Project id not valid")
            return {
                "is_success": False,
                "error": f'Project id not available'
            }
        print(f'delete project - project id: {project_id}')
        _studio_client = studio.StudioClient(
            self.encoded_key_secret, self.layernext_url)
        try:
            response = _studio_client.delete_project(project_id)
            return response
        except Exception as e:
            print("An exception occurred", format(e))
            return {
                "is_success": False,
                "error": f'An exception occurred: {format(e)}'
            }

    """
    update the labels of studio project
    """

    def __update_labels_to_studio_project(self, project_id: str, add_list, remove_list):
        # print(f'add labels to project - project id: {project_id}, add label list: {add_list}, remove label list: {remove_list}')

        _studio_client = studio.StudioClient(
            self.encoded_key_secret, self.layernext_url)

        response = _studio_client.update_labels_to_project(
            project_id, add_list, remove_list)

        return response

    """
    add the labels of studio project
    """

    def add_labels_to_studio_project(self, project_id: str, add_list):
        print(
            f'add labels to project - project id: {project_id}, label list: {add_list}')
        try:
            response = self.__update_labels_to_studio_project(
                project_id, add_list, [])
            print('response: ', response)
            return response
        except Exception as e:
            print("An exception occurred", format(e))
            return {
                "is_success": False,
                "error": f'An exception occurred: {format(e)}'
            }

    """
    remove the labels of studio project
    """

    def remove_labels_to_studio_project(self, project_id: str, remove_list):
        print(
            f'remove labels to project - project id: {project_id}, label list: {remove_list}')

        try:
            response = self.__update_labels_to_studio_project(
                project_id, [], remove_list)
            print('response: ', response)
            return response
        except Exception as e:
            print("An exception occurred", format(e))
            return {
                "is_success": False,
                "error": f'An exception occurred: {format(e)}'
            }

    """
    Get list of studio project
    """

    def get_annotation_project_list(self):
        _studio_client = studio.StudioClient(
            self.encoded_key_secret, self.layernext_url)
        return _studio_client.studio_project_list()

    """
    create dataset from search objects
    create dataset from search objects
    @param dataset_name - name of the dataset
    @param collection_id - collection_id for search objects
    @param query - query for search objects
    @param filter - filter for filter objects - {
        "annotation_types": ["human", "raw", "machine"],
        "from_date": "", "to_date": ""
    }
    @param object_type - object_type for search objects
    @param object_list - object_list for search objects
    @param split_info - dataset split information - {
            "train": number,
            "test": number,
            "validation": number
        }
    @param labels - dataset selected labels
    @param export_types - dataset selected export types
    @operation_list - operation id list
    @augmentation_list - augmentation list
    """

    def __create_dataset(
            self,
            dataset_name: str,
            collection_id: str,
            query: str,
            filter,
            object_type,
            object_list,
            split_nfo,
            labels,
            export_types,
            operation_list,
            augmentation_list):
        print(f'create dataset - dataset name: {dataset_name}')

        if dataset_name == "" or dataset_name == None:
            print(f'Invalid dataset name "{dataset_name}"')
            return {
                "is_success": False,
                "error": "Invalid dataset name"
            }
        if object_type.lower() == "image":
            object_type = ObjectType.IMAGE.value
        elif object_type.lower() == "image_collection":
            object_type = ObjectType.IMAGE_COLLECTION.value
        elif object_type.lower() == "dataset":
            object_type = ObjectType.DATASET.value
        else:
            print("Invalid content type - should be either 'image'")
            return {
                "is_success": False,
                "collection_id": None
            }

        filter = self.__create_filter(filter)

        _dataset_client = dataset.DatasetClient(
            self.encoded_key_secret, self.layernext_url)

        _datalake_client = datalake.DatalakeClient(
            self.encoded_key_secret, self.layernext_url)

        response = _datalake_client.get_selection_id(
            collection_id, query, filter, object_type, object_list)
        print('selection id: ', response)
        get_selection_id_success = True
        if ("isSuccess" in response):
            if (response["isSuccess"] == False):
                get_selection_id_success = False
        if get_selection_id_success == True:
            dataset_response = _dataset_client.create_dataset(
                dataset_name,
                response['selectionTag'],
                split_nfo,
                labels,
                export_types,
                operation_list,
                augmentation_list)
            print('dataset_response: ', dataset_response)
            return dataset_response
        else:
            print('dataset_response: ', {
                "isSuccess": False
            })
            return {
                "isSuccess": False
            }

    """
    create dataset from search objects on collection
    @param dataset_name - name of the dataset
    @param collection_id - collection_id for search objects
    @param split_info - dataset split information - {
            "train": number,
            "test": number,
            "validation": number
        }
    @param labels - dataset selected labels
    @param export_types - dataset selected export types
    @param query - query for search objects
    @param filter - filter for filter objects - {
        "annotation_types": ["human", "raw", "machine"],
        "from_date": "", "to_date": ""
    }
    @operation_list - operation id list
    @augmentation_list - augmentation list
    """

    def create_dataset_from_collection(
            self,
            dataset_name: str,
            collection_id: str,
            split_info,
            labels=[],
            export_types=[],
            query: str = "",
            filter={},
            operation_list=None,
            augmentation_list=None):

        if (collection_id == None):
            return {
                "isSuccess": False,
                "error": 'collection id must be provided'
            }

        object_type_object = self.__get_object_type_by_id(collection_id)
        object_type = None
        if "objectType" in object_type_object:
            object_type = object_type_object["objectType"]

        if object_type == None:
            print('object type cannot find for collection id')
            return {
                "isSuccess": False,
                "error": 'object type cannot find for collection id'
            }
        if object_type == ObjectType.IMAGE_COLLECTION.value:
            object_type = "image"
        else:
            return {
                "isSuccess": False,
                "error": 'collection type is not acceptable for create dataset, only image collections are valid'
            }

        return self.__create_dataset(
            dataset_name,
            collection_id,
            query,
            filter,
            object_type,
            [],
            split_info,
            labels,
            export_types,
            operation_list,
            augmentation_list)

    """
    create dataset from search objects on datalake
    @param dataset_name - name of the dataset
    @param split_info - dataset split information - {
            "train": number,
            "test": number,
            "validation": number
        }
    @param labels - dataset selected labels
    @param export_types - dataset selected export types
    @param content_type - content_type for search objects
    @param query - query for search objects
    @param filter - filter for filter objects - {
        "annotation_types": ["human", "raw", "machine"],
        "from_date": "", "to_date": ""
    }
    @operation_list - operation id list
    @augmentation_list - augmentation list
    """

    def create_dataset_from_datalake(
            self,
            dataset_name: str,
            split_info,
            labels=[],
            export_types=[],
            item_type="image",
            query: str = "",
            filter={},
            operation_list=None,
            augmentation_list=None):

        return self.__create_dataset(
            dataset_name,
            None,
            query,
            filter,
            item_type,
            [],
            split_info,
            labels,
            export_types,
            operation_list,
            augmentation_list)

    """
    update dataset from search objects
    @param version_id - id of the base version
    @param collection_id - collection_id for search objects
    @param query - query for search objects
    @param filter - filter for filter objects - {
        "annotation_types": ["human", "raw", "machine"],
        "from_date": "", "to_date": ""
    }
    @param object_type - object_type for search objects
    @param object_list - object_list for search objects
    @param split_info - dataset split information - {
            "train": number,
            "test": number,
            "validation": number
        }
    @param labels - dataset selected labels
    @param export_types - dataset selected export types
    @param is_new_version_required - whether creating new version (True) or update existing version (False)
    @operation_list - operation id list
    @augmentation_list - augmentation list
    """

    def __update_dataset_version(
            self,
            version_id: str,
            collection_id: str,
            query: str,
            filter,
            object_type,
            object_list,
            split_info,
            labels,
            export_types,
            is_new_version_required,
            operation_list,
            augmentation_list):
        print(f'create dataset - dataset version id: {version_id}')

        if object_type.lower() == "image":
            object_type = ObjectType.IMAGE.value
        elif object_type.lower() == "image_collection":
            object_type = ObjectType.IMAGE_COLLECTION.value
        elif object_type.lower() == "dataset":
            object_type = ObjectType.DATASET.value
        else:
            print("Invalid content type - should be either 'image'")
            return {
                "is_success": False
            }

        filter = self.__create_filter(filter)

        _dataset_client = dataset.DatasetClient(
            self.encoded_key_secret, self.layernext_url)

        _datalake_client = datalake.DatalakeClient(
            self.encoded_key_secret, self.layernext_url)

        response = _datalake_client.get_selection_id(
            collection_id, query, filter, object_type, object_list)
        print('selection id: ', response)
        get_selection_id_success = True
        if ("isSuccess" in response):
            if (response["isSuccess"] == False):
                get_selection_id_success = False
        if get_selection_id_success == True:
            dataset_response = _dataset_client.update_dataset_version(
                version_id,
                response['selectionTag'],
                split_info,
                labels,
                export_types,
                is_new_version_required,
                operation_list,
                augmentation_list)
            print('dataset_response: ', dataset_response)
            return dataset_response
        else:
            print('dataset_response: ', {
                "isSuccess": False
            })
            return {
                "isSuccess": False
            }

    """
    update dataset from search objects on collection
    @param version_id - id of the base version
    @param collection_id - collection_id for search objects
    @param split_info - dataset split information - {
            "train": number,
            "test": number,
            "validation": number
        }
    @param labels - dataset selected labels
    @param export_types - dataset selected export types
    @param query - query for search objects
    @param filter - filter for filter objects - {
        "annotation_types": ["human", "raw", "machine"],
        "from_date": "", "to_date": ""
    }
    @param is_new_version_required - whether creating new version (True) or update existing version (False)
    @operation_list - operation id list
    @augmentation_list - augmentation list
    """

    def update_dataset_version_from_collection(
            self,
            version_id: str,
            collection_id: str,
            split_info,
            labels=[],
            export_types=[],
            query: str = "",
            filter={},
            is_new_version_required=False,
            operation_list=None,
            augmentation_list=None):

        if (collection_id == None or collection_id == ""):
            return {
                "isSuccess": False,
                "error": 'collection id must be provided'
            }
        if (version_id == None or version_id == ""):
            return {
                "isSuccess": False,
                "error": 'version id must be provided'
            }

        object_type_object = self.__get_object_type_by_id(collection_id)
        object_type = None
        if "objectType" in object_type_object:
            object_type = object_type_object["objectType"]

        if object_type == None:
            print('object type cannot find for collection id')
            return {
                "isSuccess": False,
                "error": 'object type cannot find for collection id'
            }
        if object_type == ObjectType.IMAGE_COLLECTION.value:
            object_type = "image"
        else:
            return {
                "isSuccess": False,
                "error": 'collection type is not acceptable for create dataset, only image collections are valid'
            }

        return self.__update_dataset_version(
            version_id,
            collection_id,
            query,
            filter,
            object_type,
            [],
            split_info,
            labels,
            export_types,
            is_new_version_required,
            operation_list,
            augmentation_list)

    """
    update dataset from search objects on datalake
    @param version_id - id of the base version
    @param split_info - dataset split information - {
            "train": number,
            "test": number,
            "validation": number
        }
    @param labels - dataset selected labels
    @param export_types - dataset selected export types
    @param item_type - content_type for search objects
    @param query - query for search objects
    @param filter - filter for filter objects - {
        "annotation_types": ["human", "raw", "machine"],
        "from_date": "", "to_date": ""
    }
    @param is_new_version_required - whether creating new version (True) or update existing version (False)
    @operation_list - operation id list
    @augmentation_list - augmentation list
    """

    def update_dataset_version_from_datalake(
            self,
            version_id: str,
            split_info,
            labels=[],
            export_types=[],
            item_type: str = "image",
            query: str = "",
            filter={},
            is_new_version_required=False,
            operation_list=None,
            augmentation_list=None):

        if (version_id == None or version_id == ""):
            return {
                "isSuccess": False,
                "error": 'version id must be provided'
            }
        return self.__update_dataset_version(
            version_id,
            None,
            query,
            filter,
            item_type,
            [],
            split_info,
            labels,
            export_types,
            is_new_version_required,
            operation_list,
            augmentation_list)

    """
    delete dataset version
    @param version_id - id of the deleting version
    """

    def delete_dataset_version(self, version_id: str):
        print(f'delete dataset version - dataset version id: {version_id}')

        _dataset_client = dataset.DatasetClient(
            self.encoded_key_secret, self.layernext_url)
        response = _dataset_client.delete_dataset_version(version_id)
        print(response)
        return response

    """
    Get list of all system labels
    """

    def get_all_labels(self):
        _datalake_client = datalake.DatalakeClient(
            self.encoded_key_secret, self.layernext_url)
        return _datalake_client.get_system_labels()

    """
    Get list of system labels that attached to given group
    """

    def get_labels_in_group(self, group_id):
        _datalake_client = datalake.DatalakeClient(
            self.encoded_key_secret, self.layernext_url)
        return _datalake_client.get_system_labels(group_id)

    """
    Create a group from labels (label keys)
    """

    def create_label_group(self, group_name, label_ids):
        _datalake_client = datalake.DatalakeClient(
            self.encoded_key_secret, self.layernext_url)
        return _datalake_client.create_label_group(group_name, label_ids)

    """
    Attach labels to a group
    """

    def attach_labels_to_group(self, group_id, label_ids):
        _datalake_client = datalake.DatalakeClient(
            self.encoded_key_secret, self.layernext_url)
        return _datalake_client.attach_labels_to_group(group_id, label_ids)

    """
    Detach labels from group
    """

    def detach_labels_from_group(self, group_id, label_ids):
        _datalake_client = datalake.DatalakeClient(
            self.encoded_key_secret, self.layernext_url)
        return _datalake_client.detach_labels_from_group(group_id, label_ids)

    """
    List All label groups
    """

    def get_all_label_groups(self):
        _datalake_client = datalake.DatalakeClient(
            self.encoded_key_secret, self.layernext_url)
        return _datalake_client.get_all_label_groups()

    """
    Attach label group to annotation project
    """

    def attach_label_group_to_annotation_project(self, project_id, group_id):
        _studio_client = studio.StudioClient(
            self.encoded_key_secret, self.layernext_url)
        return _studio_client.project_set_label_group(project_id, group_id)

    """
    Deprecated
    Download project annotations
    @param project_id - id of annotation project
    @param task_status_list - (Optional): To filter by task status - default: []
    @param is_annotated_only - (Optional): If True, then only the annotated images are downloaded - default: False
    @param custom_download_path - (Optional): To download data to required location instead of current directory - default: None
    """

    def download_project_annotations(self, project_id, task_status_list: list = [], is_annotated_only: bool = False, custom_download_path: str = None):

        print('This method is now deprecated and please switch to "download_annotation_projects" function as this will be removed in future')
        # init dataset client
        _dataset_client = dataset.DatasetClient(
            self.encoded_key_secret, self.layernext_url, custom_download_path)

        project_id_list = []
        project_id_list.append(project_id)
        # start download
        _dataset_client.download_annotations_for_project_v2(
            project_id_list, task_status_list, is_annotated_only)

    """
    Download multiple project annotations
    @param project_id_list - id list of annotation projects
    @param task_status_list - (Optional): To filter by task status - default: []
    @param is_annotated_only - (Optional): If True, then only the annotated images are downloaded - default: False
    @param custom_download_path - (Optional): To download data to required location instead of current directory - default: None
    """

    def download_annotation_projects(self, project_id_list, task_status_list: list = [], is_annotated_only: bool = False, custom_download_path: str = None):

        if task_status_list == None:
            task_status_list = []

        # init dataset client
        _dataset_client = dataset.DatasetClient(
            self.encoded_key_secret, self.layernext_url, custom_download_path)

        # start download
        _dataset_client.download_annotations_for_project_v2(
            project_id_list, task_status_list, is_annotated_only)

    """
    Wait until job complete
    @param job_id - id of the relevant job
    """

    def wait_for_job_complete(self, job_id: str):
        _datalake_client = datalake.DatalakeClient(
            self.encoded_key_secret, self.layernext_url)
        return _datalake_client.wait_for_job_complete(job_id)

    """
    Get downloadable url of a file in Data Lake
    @param file_key - File path in Data Lake
    """

    def get_downloadable_url(self, file_key: str):
        _datalake_client = datalake.DatalakeClient(
            self.encoded_key_secret, self.layernext_url)
        return _datalake_client.get_file_download_url(file_key)

    """
    Upload metadata data from a json file
    """

    def upload_metadata_for_files(
            self,
            collection_base_path: str,
            content_type: str,
            json_data_file_path: str,
    ):
        # init datalake client
        _datalake_client = datalake.DatalakeClient(
            self.encoded_key_secret, self.layernext_url)

        if content_type.lower() == "image":
            collection_type = MediaType.IMAGE.value
        elif content_type.lower() == "video":
            collection_type = MediaType.VIDEO.value
        elif content_type.lower() == "other":
            collection_type = MediaType.OTHER.value
        else:
            raise Exception("Invalid content type")

        _datalake_client.upload_metadata_from_json(
            collection_base_path,
            collection_type,
            json_data_file_path
        )

    """
    Upload metadata data from a metadata object
    """

    def upload_metadata_for_collection(
            self,
            collection_name: str,
            content_type: str,
            metadata_obj: dict,
            is_apply_to_all_files=True):

        _datalake_client = datalake.DatalakeClient(
            self.encoded_key_secret, self.layernext_url)

        if content_type.lower() == "image":
            collection_type = MediaType.IMAGE.value
        elif content_type.lower() == "video":
            collection_type = MediaType.VIDEO.value
        elif content_type.lower() == "other":
            collection_type = MediaType.OTHER.value
        else:
            raise Exception("Invalid content type")

        _datalake_client.upload_metadata_from_metaObject(
            collection_name,
            collection_type,
            metadata_obj,
            is_apply_to_all_files
        )



from layernext.datalake.constants import MULTI_PART_UPLOAD_CHUNK_SIZE, MediaType
from layernext.datalake.storage_interface import StorageInterface


class StorageUpload:

    def __init__(self, client, storage_type, collection_name, upload_id, upload_type, test_write_progress, add_fail_files, application_code):
        self._client = client
        self.collectionName = collection_name
        self.uploadId = upload_id
        self.write_progress = test_write_progress
        self.add_fail_files = add_fail_files
        self.application_code = application_code
        self.file_id = ""
        self.file_key = ""
        self.storage_type = storage_type
        self.upload_type = upload_type

    """"
    Initialize multipart upload
    """

    def initialize_multipart_upload(self, file_name: str):
        payload = {
            "fileName": file_name,
            "collectionName": self.collectionName,
            "uploadId": self.uploadId,
            "applicationCode": self.application_code
        }
        multipart_init_res = self._client.datalake_interface.get_file_id_and_key(payload)
        return multipart_init_res

    """"
    Multipart upload
    """

    def multi_part_upload(self, sub_list):

        for file in sub_list:
            uploaded_parts_arr = []
            for uploaded_part in file["uploaded_parts_arr"]:
                uploaded_parts_arr.append(uploaded_part)

            if(file["file_id"] == None):
                '''Get file id and key'''
                multipart_init_res = self.initialize_multipart_upload(file["key"])

                if multipart_init_res["isSuccess"]:
                    self.file_id = multipart_init_res["fileId"]
                    self.file_key = multipart_init_res["fileKey"]
                    #If isExisting is present and is true, then skip all operations to file because its already uploaded to datalake
                    if "isExisting" in multipart_init_res and multipart_init_res["isExisting"]:
                        self.write_progress(uploaded_chunk_count=file["part_count"])
                        print(f'\nFile {self.file_key} already exists in datalake. Skipping upload')
                        continue
                else:
                    file["is_all"] = True
                    self.add_fail_files(file, None, uploaded_parts_arr, file["part_numbers"])
                    continue
            else:
                self.file_id = file["file_id"]
                self.file_key = file["file_key"]

            #Set content type based on upload type
            content_type = "application/octet-stream"
            if(self.upload_type == MediaType.IMAGE.value):
                content_type = "image/jpeg"
            elif(self.upload_type ==  MediaType.VIDEO.value):
                content_type = "video/mp4"
            
            pre_signed_url_pay_load = {
                "fileId": self.file_id,
                "fileKey": self.file_key,
                "parts": file["part_count"],
                "contentType": content_type,
            }
            #print('Uploading file: ' + self.file_key + ' with ' + str(file["part_count"]) + ' parts')

            """"Get pre signed url"""
            pre_signed_url_response = self._client.datalake_interface.get_pre_signed_url(pre_signed_url_pay_load)
            url_array = []
            if pre_signed_url_response["isSuccess"]:
                if file["is_all"]:
                    if len(file["part_numbers"]) == 0:
                        url_array = pre_signed_url_response["parts"]
                    else:
                        for part_num in file["part_numbers"]:
                            index = next((index for (index, part) in enumerate(pre_signed_url_response["parts"]) if part["PartNumber"] == part_num), None)
                            url_array.append(pre_signed_url_response["parts"][index])
            else:
                file["is_all"] = True
                file["file_id"] = self.file_id
                file["file_key"] = self.file_key
                self.add_fail_files(file, None, uploaded_parts_arr, file["part_numbers"])
                continue

            is_upload_success = True
            storage_interface = StorageInterface(self.storage_type, url_array)
            # uploaded_parts_arr = []
            # if file["is_all"] == True:
            finalize_url = ""
            for part in url_array:

                upload_s3_response = {
                    "isSuccess": False
                }
                finalize_url = part["signedUrl"]

                # if one chunk failed skip the proceeding chunks of this file and add to failed list
                if is_upload_success == True:
                    

                    """"Upload s3 file"""
                    next_byte = MULTI_PART_UPLOAD_CHUNK_SIZE * (part["PartNumber"] - 1)
                    #Count of bytes to read for each chunk, in case of last part, read only remaining bytes
                    chunk_id = part["PartNumber"] - 1
                    max_read_count = MULTI_PART_UPLOAD_CHUNK_SIZE if chunk_id < file["part_count"] - 1 else file["size"] - next_byte
                    #url, path, content_type, chunk_id, file_size, start_byte, read_bytes
                    upload_s3_response = storage_interface.upload_to_storage(part["signedUrl"], file["path"], content_type, chunk_id, file["size"], next_byte, max_read_count)

                if not upload_s3_response["isSuccess"]:
                    file["is_all"] = True
                    file["file_id"] = self.file_id
                    file["file_key"] = self.file_key
                    self.add_fail_files(file, part["PartNumber"], uploaded_parts_arr, None)

                    # if one chunk fail to upload set is_upload_success flag False
                    is_upload_success = False
                    # break
                    # print(f'part number: {part["PartNumber"]} failed')
                else:
                    # print(f'part number: {part["PartNumber"]} success')
                    uploaded_parts_arr.append({
                        "PartNumber": part["PartNumber"],
                        "ETag": upload_s3_response["e_tag"]
                    })

                    #Write progress only if its not last part
                    if part["PartNumber"] < file["part_count"]:
                        self.write_progress(uploaded_file_count=0)
                
            if not is_upload_success:
                # self.add_fail_files(file["key"])
                # #print('Failed to upload file: ' + file["key"])
                continue

            """"Finalize multipart upload"""
            finalize_payload = {
                "fileId": self.file_id,
                "fileKey": self.file_key,
                "parts": uploaded_parts_arr,
                "uploadId": self.uploadId,
                "finalizeUrl": finalize_url
            }

            finalize_re = self._client.datalake_interface.finalize_upload(finalize_payload)

            if finalize_re["isSuccess"]:
                self.write_progress()
                # print(f'\nFile {self.file_key} successfully uploaded to datalake')
            else:
                # print('Failed to finalize upload file: ' + file["key"])
                file["is_all"] = False
                file["file_id"] = self.file_id
                file["file_key"] = self.file_key
                self.add_fail_files(file, None, uploaded_parts_arr, file["part_numbers"])



import datetime
import json
import uuid
from typing import TYPE_CHECKING
import time

from .constants import BOX_ANNOTATION, OPERATION_MODE_HUMAN, OPERATION_TYPE_ANNOTATION, OPERATION_MODE_AUTO, META_UPDATE_REQUEST_BATCH_SIZE, \
    POLYGON_ANNOTATION, LINE_ANNOTATION, AnnotationUploadType
from .keys import ATTRIBUTES, BUCKET_NAME, IMAGES, IMAGE, ANNOTATIONS, BBOX, BOX_BOUNDARIES_AND_DIMENSIONS, STORAGE_NAME, STORAGE_PATH, VALUE, X, Y, W, H, POINTS, LABEL, \
    LABEL_TEXT, REF, KEY, COLOR, ATTRIBUTE_VALUES, METADATA, TEXTS, TYPE, ID, SHAPE_ID, CREATED_AT, OBJECT_KEY, \
    ANNOTATION_OBJECTS, DATA, IS_USER_ANNOTATED, UPDATED_AT, LINE, POLYGON, CONFIDENCE
from .logger import get_debug_logger
from .label import Label

if TYPE_CHECKING:
    from . import DatalakeClient

annotation_logger = get_debug_logger('Annotation')


class Annotation:
    def __init__(self, client: "DatalakeClient"):
        self._client = client

    @staticmethod
    def format_bbox_annotation(annotation_object_input, annotation_object_output):
        if BBOX in annotation_object_input and len(annotation_object_input[BBOX]) == 4:
            x = annotation_object_input[BBOX][0]
            y = annotation_object_input[BBOX][1]
            width = annotation_object_input[BBOX][2]
            height = annotation_object_input[BBOX][3]

            annotation_object_output[BOX_BOUNDARIES_AND_DIMENSIONS] = {
                X: x,
                Y: y,
                W: width,
                H: height
            }
            annotation_object_output[POINTS] = [
                [x, y],
                [x + width, y],
                [x + width, y + height],
                [x, y + height]
            ]

    @staticmethod
    def format_line_annotation(annotation_object_input, annotation_object_output):
        if LINE in annotation_object_input:
            # calculate coordinates
            annotation_object_output[POINTS] = annotation_object_input[LINE]

    @staticmethod
    def format_polygon_annotation(annotation_object_input, annotation_object_output):
        if POLYGON in annotation_object_input:
            # calculate coordinates
            annotation_object_output[POINTS] = annotation_object_input[POLYGON]

    def upload_annotation_json(
            self,
            storage_base_path: str,
            operation_id: str,
            file_path: str,
            annotation_geometry: str,
            operation_mode: int,
            is_normalized: bool,
            dest_project_id: str,
            version: str,
            bucket_name: str,
            upload_type: AnnotationUploadType
    ):

        session_uuid = str(datetime.datetime.now().timestamp())
        storage_path = None

        # load json file
        f = open(file_path)
        annotation_data = json.load(f)
        f.close()

        # get labels,attributes,values in the json files as a dictionary
        if version is None:
            label_attribute_values_dict = Label.get_label_attribute_values_dict(annotation_data)
        elif version == 'v2':
            label_attribute_values_dict = Label.get_label_attribute_values_dict_v2(annotation_data)
        else:
            raise Exception(f'Invalid version: {version}')
        annotation_logger.debug('retrieving datalake label references started')
        label_references = self._client.datalake_interface.find_datalake_label_references(label_attribute_values_dict)
        annotation_logger.debug('retrieving datalake label references finished')

        # format data to call datalake operation data update API
        meta_updates_list = []
        request_batch_size = META_UPDATE_REQUEST_BATCH_SIZE
        total_images_count = len(annotation_data[IMAGES])
        uploaded_images_count = 0
        annotation_logger.debug(f'total images count with annotation data: {total_images_count}')

        for image in annotation_data[IMAGES]:
            if upload_type == AnnotationUploadType.BY_FILE_NAME:
                object_key = f'{storage_base_path}_{image[IMAGE]}'
            elif upload_type == AnnotationUploadType.BY_STORAGE_PATH:
                object_key = None
                storage_path = f'{image[IMAGE]}'
            elif upload_type == AnnotationUploadType.BY_UNIQUE_NAME:
                object_key = f'{image[IMAGE]}'
            elif upload_type == AnnotationUploadType.BY_FILE_NAME_OR_UNIQUE_NAME:
                if storage_base_path is None:
                    object_key = f'{image[IMAGE]}'
                else:
                    object_key = f'{storage_base_path}_{image[IMAGE]}'
            else:
                raise Exception(f'Invalid upload type: {upload_type}')
            
            if STORAGE_NAME in image:
                bucket_name = image[STORAGE_NAME]
            
            annotation_objects = []
            i = 1
            for annotation in image[ANNOTATIONS]:
                annotation_object = {}
                if annotation_geometry is None and TYPE not in annotation:
                    raise Exception(f'Invalid annotation data. Annotation type not found in annotation {annotation}')
                    
                if TYPE in annotation:
                    annotation_geometry = annotation[TYPE]

                # calculate coordinates and other dimensions
                if annotation_geometry == BOX_ANNOTATION:
                    self.format_bbox_annotation(annotation, annotation_object)
                elif annotation_geometry == POLYGON_ANNOTATION:
                    self.format_polygon_annotation(annotation, annotation_object)
                elif annotation_geometry == LINE_ANNOTATION:
                    self.format_line_annotation(annotation, annotation_object)
                else:
                    break

                # assign annotation geometry type, ids, timestamps, confidence, etc.
                annotation_object_uuid = uuid.uuid4()
                annotation_object[ID] = f'shape_{annotation_object_uuid}'
                annotation_object[SHAPE_ID] = i
                i += 1
                annotation_object[CREATED_AT] = {'$date': str(datetime.datetime.now())}
                annotation_object[TYPE] = annotation_geometry
                if CONFIDENCE in annotation:
                    annotation_object[CONFIDENCE] = annotation[CONFIDENCE]

                try:
                    # assign label class
                    annotation_object[LABEL] = {
                        LABEL: label_references[annotation[LABEL]][REF],
                        LABEL_TEXT: annotation[LABEL],
                        CONFIDENCE: annotation[CONFIDENCE],
                        METADATA: {},
                        KEY: label_references[annotation[LABEL]][REF],
                        COLOR: label_references[annotation[LABEL]][COLOR],
                        ATTRIBUTE_VALUES: {}
                    }
                    # assign label attribute and values
                    if version is None:
                        if METADATA in annotation:
                            for attr, val in annotation[METADATA].items():
                                try:
                                    attr_ref = label_references[annotation[LABEL]][TEXTS][attr][REF]
                                    val_ref = label_references[annotation[LABEL]][TEXTS][attr][TEXTS][val]
                                    if attr_ref not in annotation_object[LABEL][ATTRIBUTE_VALUES]:
                                        annotation_object[LABEL][ATTRIBUTE_VALUES][attr_ref] = []
                                    annotation_object[LABEL][ATTRIBUTE_VALUES][attr_ref].append(
                                        {
                                            VALUE: val_ref, 
                                            CONFIDENCE: 1 if operation_mode == OPERATION_MODE_HUMAN else annotation[CONFIDENCE]
                                        }
                                    )
                                except (TypeError, KeyError, Exception) as te:
                                    print(f"An Error Occurred at attribute value reference finding for object_key: "
                                        f"{object_key}", te)
                                    continue
                    elif version == 'v2':
                        if ATTRIBUTES in annotation:
                            if METADATA in annotation:
                                annotation_object[LABEL][METADATA] = annotation[METADATA]
                            for attr, val_array in annotation[ATTRIBUTES].items():
                                try:
                                    attr_ref = label_references[annotation[LABEL]][TEXTS][attr][REF]
                                    
                                    for val_obj in val_array:
                                        val_ref = label_references[annotation[LABEL]][TEXTS][attr][TEXTS][val_obj[VALUE]]
                                        val_obj[VALUE] = val_ref
                                    annotation_object[LABEL][ATTRIBUTE_VALUES][attr_ref] = val_array
                                except (TypeError, KeyError, Exception) as te:
                                    print(f"An Error Occurred at attribute value reference finding for object_key: "
                                        f"{object_key}", te)
                                    continue
                    else:
                        raise Exception(f'Invalid version: {version}')

                    annotation_objects.append(annotation_object)
                except (TypeError, KeyError, Exception) as te:
                    print(f"An Error Occurred at label class reference finding for object_key: {object_key}", te)
                    continue

            meta_update_data = {
                ANNOTATION_OBJECTS: annotation_objects,
                IS_USER_ANNOTATED: False,
            }
            meta_updates = {
                OBJECT_KEY: object_key,
                STORAGE_PATH: storage_path,
                BUCKET_NAME: bucket_name,
                DATA: meta_update_data
            }
            meta_updates_list.append(meta_updates)

            # call datalake operation data update API if batch size equals,
            # this is to reduce memory consumption & to stop request body size exceeding
            if len(meta_updates_list) == request_batch_size:
                uploaded_images_count = uploaded_images_count + len(meta_updates_list)
                annotation_logger.debug(f'uploading annotation data of batch size: {len(meta_updates_list)}')
                meta_update_response = self._client.datalake_interface.upload_metadata_updates(
                    meta_updates_list,
                    OPERATION_TYPE_ANNOTATION,
                    operation_mode,
                    operation_id,
                    is_normalized,
                    session_uuid,
                    total_images_count,
                    uploaded_images_count,
                    dest_project_id
                )
                annotation_logger.debug(f'annotation data uploaded images count: {uploaded_images_count}')

                meta_updates_list = []

        # call datalake operation data update API
        # this will handle final batch
        uploaded_images_count = uploaded_images_count + len(meta_updates_list)
        annotation_logger.debug(f'uploading annotation data of batch size: {len(meta_updates_list)}')
        meta_update_response = self._client.datalake_interface.upload_metadata_updates(
            meta_updates_list,
            OPERATION_TYPE_ANNOTATION,
            operation_mode,
            operation_id,
            is_normalized,
            session_uuid,
            total_images_count,
            uploaded_images_count,
            dest_project_id
        )
        annotation_logger.debug(f'annotation data uploaded images count: {uploaded_images_count}')

    def remove_collection_annotations(self, collection_id, model_run_id):
        session_uuid = str(datetime.datetime.now().timestamp())
        meta_update_response = self._client.datalake_interface.remove_modelrun_collection_annotation(collection_id, model_run_id, session_uuid)
        print('delete annotation status: ',meta_update_response)
        return meta_update_response


import os
import sys
import threading
import time
import copy
from multiprocessing.pool import ThreadPool

from layernext.datalake.constants import MAX_UPLOAD_RETRY_COUNT, MediaType, FILE_UPLOAD_THREADS, SUB_FILE_LENGTH, MULTI_PART_UPLOAD_CHUNK_SIZE
from layernext.datalake.storage_upload import StorageUpload

imageExtensionList = ['jpg', 'jpeg', 'png']
videoExtensionList = ['mp4', 'mkv']


class FileUpload:
    def __init__(self, client: "DatalakeClient", application_code = None):
        self._client = client
        self.folder_upload = True
        self.path = ""
        self.upload_type = MediaType.IMAGE.value
        self.payload = ''
        self.objectKeyList = []
        self.file_list = []
        self.uploadId = ""
        self.collectionName = ''
        self.collectionId = ""
        self.key_name_array = []
        #Set of failed upload files as key-value map of file name - path
        self.fail_upload_set = []
        self.folder_path = ""
        self.total_keys = 0
        self.progress = 0
        self.count = 0
        self.failed_upload_count = 0
        self.application_code = application_code
        self.count_lock = threading.Lock()
        self.total_chunk_count = 0
        self.uploaded_chunk_count = 0

    '''
    Split object key list
    '''

    def split_object_key_list(self, input_list, sub_size):
        sub_list = [input_list[i:i + sub_size] for i in range(0, len(self.key_name_array), sub_size)]
        return sub_list

    ''''
    Check media type
    '''

    def check_media_type(self, extension):
        #Convert extension to lower case for case insensitive comparison
        extension = extension.lower()
        if self.upload_type == MediaType.IMAGE.value and extension in imageExtensionList:
            return True
        elif self.upload_type == MediaType.VIDEO.value and extension in videoExtensionList:
            return True
        elif self.upload_type == MediaType.OTHER.value:
            return True
        else:
            return False

    ''''
    Get media file list
    '''

    def get_media_files_list(self):
        dir_list = []

        if self.folder_upload:
            dir_list = os.listdir(self.path)
            self.folder_path = self.path
        else:
            path_array = self.path.split("/")
            dir_list.append(path_array[-1])
            path_array.pop()

            index = 0
            for path_component in path_array:
                if index != 0:
                    self.folder_path += "/"

                self.folder_path += path_component

                index += 1

        # Filter according to upload type
        for file_name in dir_list:
            extension = file_name.split(".")[-1]

            if self.check_media_type(extension):
                #Get file size in bytes
                file_full_path = self.folder_path + "/" + file_name
                file_size = os.path.getsize(file_full_path)
                #Determine multi - part upload chunk count - take ceiling of file size / chunk size
                chunk_count = int(file_size / MULTI_PART_UPLOAD_CHUNK_SIZE) + 1
                self.total_chunk_count += chunk_count
                self.objectKeyList.append(file_name)
                self.file_list.append(file_name)
                self.key_name_array.append({
                    "key": file_name,
                    "path": file_full_path,
                    "part_count": chunk_count,
                    "size": file_size,
                    "is_all": True,
                    "part_numbers": [],
                    "uploaded_parts_arr": [],
                    "file_id": None,
                    "file_key": None
                })
                #print("File name: " + file_name + " File size: " + str(file_size) + " Chunk count: " + str(chunk_count))
        self.total_keys = len(self.key_name_array)

    '''
    Write progress
    '''

    def write_progress(self, count=True, uploaded_file_count=1, uploaded_chunk_count=1):
        if count:
            self.count_lock.acquire()
            try:
                self.count += uploaded_file_count
                self.uploaded_chunk_count += uploaded_chunk_count
                self.progress = 100 * (self.uploaded_chunk_count / self.total_chunk_count)
                sys.stdout.write(
                    "\r" + "upload files: " + str(self.count) + f"/{self.total_keys}" + "     " + "progress: " +
                    str(round(self.progress, 2)) + " %")
                sys.stdout.flush()
            finally:
                self.count_lock.release()
        else:
            sys.stdout.write(
                "\r" + "upload files: " + str(self.count) + "     " + "progress: " +
                str(round(self.progress, 2)) + " %")
            sys.stdout.flush()

    ''''
    Add upload failed files into an array
    '''

    def add_fail_files(self, file, part_number = None, uploaded_parts_arr = [], part_number_array = []):
        self.count_lock.acquire()
        try:

            index = next((index for (index, _file) in enumerate(self.fail_upload_set) if _file["key"] == file["key"]), None)

            if(index != None):
                if(part_number != None):
                    self.fail_upload_set[index]["part_numbers"].append(part_number)
                elif(part_number_array != None):
                    if(len(part_number_array) > 0):
                        self.fail_upload_set[index]["part_numbers"] = part_number_array
                if(uploaded_parts_arr != None):
                    for uploaded_part in uploaded_parts_arr:
                        index_upload = next((index for (index, _uploaded_part) in enumerate(uploaded_parts_arr) if _uploaded_part["PartNumber"] == uploaded_part["PartNumber"]), None)
                        if(index_upload == None):
                            self.fail_upload_set[index]["uploaded_parts_arr"].append(uploaded_part)

            else:
                # file_name = object_key.split("/")[-1]
                file["part_numbers"] = []
                if(part_number != None):
                    file["part_numbers"].append(part_number)
                elif(part_number_array != None):
                    if(len(part_number_array) > 0):
                        file["part_numbers"] = part_number_array

                if(uploaded_parts_arr != None):
                    file["uploaded_parts_arr"] = uploaded_parts_arr
                self.fail_upload_set.append(file)
            #Set failed count from the key count of fail_upload_set
            self.failed_upload_count = len(self.fail_upload_set)
        finally:
            self.count_lock.release()

    ''''
    Parallel upload
    '''

    def parallel_upload(self, sublist):
        _upload = StorageUpload(self._client, self.storage_type, self.collectionName, self.uploadId, self.upload_type, self.write_progress, self.add_fail_files, self.application_code)
        _upload.multi_part_upload(sublist)

    ''''
    File upload initiate     
    '''

    def file_upload_initiate(self, path, collection_type, collection_name, meta_data_object, meta_data_override, storage_prefix_path):
       
        self.write_progress(False)

        self.path = path
        self.payload = {
            "collectionType": collection_type,
            "collectionName": collection_name,
            "metaDataObject": meta_data_object,
            "isOverrideMetaData": meta_data_override,
            "applicationCode": self.application_code,
            "storagePrefixPath": storage_prefix_path
        }
        self.upload_type = collection_type
        self.collectionName = collection_name

        if not os.path.exists(self.path):
            raise Exception("Path does not exists")
        elif os.path.isdir(path):
            self.folder_upload = True
        elif os.path.isfile(path):
            self.folder_upload = False

        """ Get media file list"""
        self.get_media_files_list()
        self.payload["objectKeyList"] = self.objectKeyList

        """Upload metadata in collection"""
        upload_metadata_response = self._client.datalake_interface.upload_metadata_collection(self.payload)

        if not upload_metadata_response["isSuccess"]:
            msg = upload_metadata_response["message"]
            raise Exception("Can not upload meta data | " + msg )

        self.storage_type = upload_metadata_response["storageType"]
        self.uploadId = upload_metadata_response["uploadId"]
        self.collectionId = upload_metadata_response["collectionId"]

        sub_list = self.split_object_key_list(self.key_name_array, SUB_FILE_LENGTH)

        process = ThreadPool(FILE_UPLOAD_THREADS)
        process.map(self.parallel_upload, sub_list)
        process.close()
        process.join()

        print(f"\n\nUpload failed count/total : {self.failed_upload_count}/{self.total_keys}")  
        retry_count = 0

        while self.failed_upload_count > 0 and retry_count < MAX_UPLOAD_RETRY_COUNT:
            print(f'Retrying {self.failed_upload_count} failed uploads.....')
            time.sleep(30)
            #Retry with half no of threads
            #Copy failed set to temp array and reset original
            temp_failed_arr = []
            for file in self.fail_upload_set:
                temp_failed_arr.append(file)
            #fail_sub_list = self.split_object_key_list(temp_failed_arr, SUB_FILE_LENGTH)
            self.fail_upload_set = []
            self.failed_upload_count = 0
            retry_count += 1
            print('Retry count: ' + str(retry_count) + ' of ' + str(MAX_UPLOAD_RETRY_COUNT)) 
            self.parallel_upload(temp_failed_arr)
            time.sleep(1)

        print('Finished processing failed uploads')
        #if self.progress != 0:
        complete_res = self._client.datalake_interface.complete_collection_upload(self.uploadId)
        if complete_res["isSuccess"]:
            print("\nComplete upload")
        else:
            print("\nError In complete upload")

        if len(self.fail_upload_set) != 0:
            print(f"\n\nupload failed files: {self.failed_upload_count}/{self.total_keys}")
            for file_name in self.fail_upload_set:
                print(f'{file_name["key"]}')
        
        return {
            "is_success": complete_res["isSuccess"],
            "job_id": upload_metadata_response["jobId"],
            "collection_id": self.collectionId
        }

    def get_upload_status(self, collection_name):
        if(collection_name == None):
            print('Invalid collection name')
        else:
            progress_response = self._client.datalake_interface.get_upload_status(collection_name)
            return progress_response


#abstract class to represent a the client in upload data to storage
class StorageClient:
    def upload_chunk(self, url, chunk_id, chunk_data, content_type, content_range, read_bytes):
        return {
            "isSuccess": True,
            "part_id": str(chunk_id)
        }

    




import codecs
import os

from setuptools import find_packages, setup

here = os.path.abspath(os.path.dirname(__file__))

with codecs.open(os.path.join(here, "README.md"), encoding="utf-8") as fh:
    long_description = "\n" + fh.read()


def read(rel_path):
    with codecs.open(os.path.join(here, rel_path), 'r') as fp:
        return fp.read()


def get_version(rel_path):
    for line in read(rel_path).splitlines():
        if line.startswith('__version__'):
            delim = '"' if '"' in line else "'"
            return line.split(delim)[1]
    else:
        raise RuntimeError("Unable to find version string.")


VERSION = get_version("layernext/__init__.py")
DESCRIPTION = 'LayerNext Python SDK'
LONG_DESCRIPTION = 'Python API Client to interact with LayerNext stack'

# Setting up
setup(
    name="layernext-enterprise-beta",
    version=VERSION,
    author="LayerNext",
    author_email="<support@layernext.ai>",
    description=DESCRIPTION,
    long_description_content_type="text/markdown",
    long_description=long_description,
    packages=find_packages(
        include=['layernext', 'layernext.datalake', 'layernext.dataset', 'layernext.studio']),
    install_requires=['requests', 'uuid', 'python-dotenv', 'azure-storage-blob'],
    keywords=['python', 'datalake', 'datasetsync', 'ai',
              'annotation', 'layernext', 'layernext', 'machine learning'],
    classifiers=[
        "Development Status :: 1 - Planning",
        "Intended Audience :: Developers",
        "Programming Language :: Python :: 3",
        "Operating System :: Unix",
        "Operating System :: MacOS :: MacOS X",
        "Operating System :: Microsoft :: Windows",
    ]
)



import logging
import logging.handlers
import sys


def get_debug_logger(name):

    _logger = logging.getLogger(name)
    _logger.setLevel(logging.DEBUG)

    handler = logging.StreamHandler(sys.stdout)
    handler.setLevel(logging.DEBUG)
    formatter = logging.Formatter('%(asctime)s - %(name)s - %(levelname)s - %(message)s')
    handler.setFormatter(formatter)
    _logger.addHandler(handler)
    return _logger



from typing import TYPE_CHECKING

from .constants import LABEL_CLASS_WITH_ATTRIBUTES
from .keys import SUPERCATEGORY, NAME, LABEL_TEXT, VALUE, VALUES, VALUE_TEXT, DESCRIPTION, IMG_FILES, ATTRIBUTES, TYPE, \
    LABEL_REF, LABEL, LABEL_COLOR, COLOR, IMAGES, ANNOTATIONS, METADATA

if TYPE_CHECKING:
    from . import DatalakeClient


class Label:
    def __init__(self, client: "DatalakeClient"):
        self._client = client

    @staticmethod
    def get_label_attribute_values_dict_v2(modelrun_data):
        label_attribute_values_dict = {}

        # iterate through images
        for image in modelrun_data[IMAGES]:
            for annotation in image[ANNOTATIONS]:
                # if label class not exists in label_attribute_values_dict, then add it
                if annotation[LABEL] not in label_attribute_values_dict:
                    label_attribute_values_dict[annotation[LABEL]] = {}

                # add attributes and values to dict from metadata
                _attribute_value = label_attribute_values_dict[annotation[LABEL]]
                if ATTRIBUTES in annotation:
                    _attributes = annotation[ATTRIBUTES]
                    for attr, val in _attributes.items():
                        # if attribute not exist, then add it
                        if attr not in _attribute_value:
                            _attribute_value[attr] = []
                        # values array iterate and if value not exist, then add it
                        for _val in val:
                            if _val[VALUE] not in _attribute_value[attr]:
                                _attribute_value[attr].append(_val[VALUE])

        return label_attribute_values_dict
    
    @staticmethod
    def get_label_attribute_values_dict(modelrun_data):
        label_attribute_values_dict = {}

        # iterate through images
        for image in modelrun_data[IMAGES]:
            for annotation in image[ANNOTATIONS]:
                # if label class not exists in label_attribute_values_dict, then add it
                if annotation[LABEL] not in label_attribute_values_dict:
                    label_attribute_values_dict[annotation[LABEL]] = {}

                # add attributes and values to dict from metadata
                _attribute_value = label_attribute_values_dict[annotation[LABEL]]
                if METADATA in annotation:
                    _metadata = annotation[METADATA]
                    for attr, val in _metadata.items():
                        # if attribute not exist, then add it
                        if attr not in _attribute_value:
                            _attribute_value[attr] = []
                        # if value not exist, then add it
                        if val not in _attribute_value[attr]:
                            _attribute_value[attr].append(val)

        return label_attribute_values_dict

    def create_label_from_cocojson(self, categories):
        category_dict = {}

        for category in categories:
            if category[SUPERCATEGORY] in category_dict:
                category_dict[category[SUPERCATEGORY]].append(category[NAME])
            else:
                category_dict[category[SUPERCATEGORY]] = [category[NAME]]

        label_dict_list = []

        # convert coco categories into datalake system label type
        for key, value in category_dict.items():

            attributes = [
                {
                    LABEL_TEXT: NAME,
                    VALUES: []
                }
            ]

            for val in value:
                attributes[0][VALUES].append({
                    VALUE_TEXT: val,
                    DESCRIPTION: '',
                    IMG_FILES: []
                })

            label_dict = {
                LABEL_TEXT: key,
                DESCRIPTION: '',
                ATTRIBUTES: attributes,
                IMG_FILES: [],
                TYPE: LABEL_CLASS_WITH_ATTRIBUTES
            }

            label_dict_list.append(label_dict)

        # call datalake to create system labels
        system_label_dict = {}
        for _label in label_dict_list:
            response = self._client.datalake_interface.create_datalake_label_coco(_label)
            if response is not None:
                if LABEL_TEXT in response:
                    system_label_dict[response[LABEL_TEXT].lower()] = response

        # add system label references to coco categories
        for category in categories:
            if category[SUPERCATEGORY].lower() in system_label_dict:
                system_label = system_label_dict[category[SUPERCATEGORY].lower()]
                category[LABEL_REF] = system_label[LABEL]
                category[LABEL_COLOR] = system_label[COLOR]
                # TODO: add system label attribute references




from typing import TYPE_CHECKING

from layernext.datalake.keys import SELECTION_ID, USER_ID


if TYPE_CHECKING:
    from . import DatalakeClient

class FileTrash:
    def __init__(self, client:"DatalakeClient"):
        self._client = client
        

    def trash_files(self, selection_id, user_id='Python SDK'):
        payload = {
            SELECTION_ID: selection_id,
            USER_ID: user_id
        }

        response = self._client.datalake_interface.trash_files(payload)
        return response
        